@@ -13,47 +13,47 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import os
-
-if 'DISPLAY' in os.environ:
-    import gtk
-    import pango
-    import cairo
-    import glib
+import gtk
+import pango
+import cairo
+import glib
 
 import ninix.pix
 
 
-class Balloon:
+class Balloon(object):
 
-    def __init__(self, callback, debug=0):
-        self.callback = callback
-        self.debug = debug
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.synchronized = []
-        self.__scalling = 0
-        self.__scale = 100 # %
-        self.__use_pna = False
-        self.__alpha_channel = 1.0
         self.user_interaction = False
         self.window = []
         # create communicatebox
-        communicatebox_callback = {
-            'reset_user_interaction': self.reset_user_interaction,
-            'notify_event':  self.callback['notify_event'],
-            }
-        self.communicatebox = CommunicateBox(communicatebox_callback, debug)
+        self.communicatebox = CommunicateBox()
+        self.communicatebox.set_responsible(self.handle_request)
         # create teachbox
-        teachbox_callback = {
-            'reset_user_interaction': self.reset_user_interaction,
-            'notify_user_teach':  self.callback['notify_user_teach'],
-            }
-        self.teachbox = TeachBox(teachbox_callback, debug)
+        self.teachbox = TeachBox()
+        self.teachbox.set_responsible(self.handle_request)
         # create inputbox
-        inputbox_callback = {
+        self.inputbox = InputBox()
+        self.inputbox.set_responsible(self.handle_request)
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
             'reset_user_interaction': self.reset_user_interaction,
-            'notify_event': self.callback['notify_event'],
             }
-        self.inputbox = InputBox(inputbox_callback, debug)
+        handler = handlers.get(event)
+        if handler is None:
+            result = self.request_parent(
+                event_type, event, *arglist, **argdict)
+        else:
+            result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
 
     def reset_user_interaction(self):
         self.user_interaction = False
@@ -74,38 +74,9 @@ class Balloon:
         if len(self.window) > side:
             self.window[side].reset_text_count()
 
-    def set_use_pna(self, flag):
-        if self.__use_pna == bool(flag):
-            return
-        self.__use_pna = bool(flag)
+    def reset_balloon(self):
         for balloon_window in self.window:
-            balloon_window.set_use_pna(self.__use_pna)
-
-    def set_alpha_channel(self, alpha):
-        if not 0.1 <= alpha <= 1.0 or alpha is None:
-            alpha = 1.0
-        if self.__alpha_channel == alpha:
-            return
-        self.__alpha_channel = alpha
-        for balloon_window in self.window:
-            balloon_window.set_alpha_channel(self.__alpha_channel)
-
-    def set_scale(self, scale):
-        if self.__scale == scale:
-            return
-        self.__scale = scale # %
-        for balloon_window in self.window:
-            balloon_window.set_scale(self.__scale)
-
-    def get_scalling(self):
-        return self.__scalling
-
-    def set_scalling(self, flag):
-        if self.__scalling == flag:
-            return
-        self.__scalling = flag
-        for balloon_window in self.window:
-            balloon_window.set_scalling(self.__scalling)
+            balloon_window.reset_balloon()
 
     def create_gtk_window(self, title):
         window = ninix.pix.TransparentWindow()
@@ -145,7 +116,7 @@ class Balloon:
         communicate1 = None
         communicate2 = None
         communicate3 = None
-        for key, value in balloon.iteritems():
+        for key, value in balloon.items():
             if key in ['arrow0', 'arrow1']:
                 balloon0[key] = value
                 balloon1[key] = value
@@ -182,35 +153,26 @@ class Balloon:
         assert len(self.window) == side
         if side == 0:
             name = 'balloon.sakura'
-            id_format = 's%d'
+            id_format = 's{0:d}'
             balloon = self.balloon0
         elif side == 1:
             name = 'balloon.kero'
-            id_format = 'k%d'
+            id_format = 'k{0:d}'
             balloon = self.balloon1
         else:
-            name = 'balloon.char%d' % side
-            id_format = 'k%d'
+            name = 'balloon.char{0:d}'.format(side)
+            id_format = 'k{0:d}'
             balloon = self.balloon1
         gtk_window = self.create_gtk_window(name)
-        balloon_window_callback = self.callback.copy() ## FIXME
         balloon_window = BalloonWindow(
-            gtk_window, side, balloon_window_callback, self.desc, balloon,
-            id_format, self.debug)
+            gtk_window, side, self.desc, balloon,
+            id_format)
+        balloon_window.set_responsible(self.handle_request)
         self.window.append(balloon_window)
-        # BalloonWindow default should be same as Balloon default
-        balloon_window.set_scalling(self.__scalling)
-        balloon_window.set_scale(self.__scale)
-        balloon_window.set_use_pna(self.__use_pna)
-        balloon_window.set_alpha_channel(self.__alpha_channel)
-        balloon_window.set_fonts(self.callback['get_preference']('balloon_fonts'))
-        if side > 1:
-            balloon_window.set_balloon(0) ## FIXME
 
     def reset_fonts(self):
-        font_name = self.callback['get_preference']('balloon_fonts')
         for window in self.window:
-            window.set_fonts(font_name)
+            window.reset_fonts()
 
     def get_balloon_directory(self):
         return self.directory
@@ -221,6 +183,12 @@ class Balloon:
         else:
             return (0, 0)
 
+    def get_balloon_windowposition(self, side):
+        if len(self.window) > side:
+            return self.window[side].get_balloon_windowposition()
+        else:
+            return (0, 0)
+
     def set_balloon_default(self):
         for side in range(len(self.window)):
             self.window[side].set_balloon(0) ## FIXME
@@ -233,9 +201,9 @@ class Balloon:
         if len(self.window) > side:
             self.window[side].set_direction(dir)
 
-    def set_position(self, side, x, y):
+    def set_position(self, side, base_x, base_y):
         if len(self.window) > side:
-            self.window[side].set_position(x, y)
+            self.window[side].set_position(base_x, base_y)
 
     def get_position(self, side):
         if len(self.window) > side:
@@ -365,7 +333,7 @@ class Balloon:
     def open_teachbox(self):
         if not self.user_interaction:
             self.user_interaction = True
-            self.callback['notify_event']('OnTeachStart')
+            self.request_parent('NOTIFY', 'notify_event', 'OnTeachStart')
             self.teachbox.show()
 
     def open_inputbox(self, symbol, limittime=-1, default=None):
@@ -376,19 +344,17 @@ class Balloon:
             self.inputbox.show(default)
 
 
-class BalloonWindow:
+class BalloonWindow(object):
 
-    def __init__(self, window, side, callback, desc, balloon,
-                 id_format, debug):
+    def __init__(self, window, side, desc, balloon, id_format):
         self.window = window
         self.side = side
-        self.callback = callback
+        self.request_parent = lambda *a: None # dummy
         self.desc = desc
         self.balloon = balloon
         self.balloon_id = None
         self.id_format = id_format
         self.num = 0
-        self.debug = debug
         self.__shown = 0
         self.sstp_marker = []
         self.sstp_region = None
@@ -396,10 +362,6 @@ class BalloonWindow:
         self.images = []
         self.width = 0
         self.height = 0
-        self.__scalling = 0
-        self.__scale = 100 # %
-        self.__use_pna = False
-        self.__alpha_channel = 1.0
         self.__font_name = ''
         self.text_count = 0
         self.balloon_pixbuf = None
@@ -434,18 +396,34 @@ class BalloonWindow:
         self.text_active_color = (text_r / 255., text_g / 255., text_b / 255.)
         # initialize
         self.direction = min(side, 1) ## kluge: multi character
-        self.set_position(0, 0)
+        self.position = (0, 0)
+        self.reset_fonts()
         self.clear_text()
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    @property
+    def scale(self):
+        scalling = self.request_parent('GET', 'get_preference', 'balloon_scalling')
+        scale = self.request_parent('GET', 'get_preference', 'surface_scale')
+        return scale if scalling else 100 # %
+
+    def get_balloon_windowposition(self):
+        winpos_x = self.__get_with_scaling('windowposition.x', int, 0)
+        winpos_y = self.__get_with_scaling('windowposition.y', int, 0)
+        return winpos_x, winpos_y
+
     def get_pixbuf(self, balloon_id):
         #assert balloon_id in self.balloon
         try:
             path, config = self.balloon[balloon_id]
+            use_pna = self.request_parent('GET', 'get_preference', 'use_pna')
             pixbuf = ninix.pix.create_pixbuf_from_file(
-                path, use_pna=self.__use_pna)
+                path, use_pna=use_pna)
         except:
             return None
-        scale = self.get_scale()
+        scale = self.scale
         w = pixbuf.get_width()
         h = pixbuf.get_height()
         w = max(8, int(w * scale / 100))
@@ -453,11 +431,8 @@ class BalloonWindow:
         pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
         return pixbuf, (w, h)
 
-    def set_use_pna(self, flag):
-        self.__use_pna = flag
-        self.reset_balloon()
-
-    def set_fonts(self, font_name):
+    def reset_fonts(self):
+        font_name = self.request_parent('GET', 'get_preference', 'balloon_fonts')
         if self.__font_name == font_name:
             return
         self.font_desc = pango.FontDescription(font_name)
@@ -468,14 +443,14 @@ class BalloonWindow:
                 ['font.height', 'font.size'], int, default_size)
             pango_size = size * 3 / 4 # convert from Windows to GTK+
             pango_size *= pango.SCALE
-        scale = self.get_scale()
+        scale = self.scale
         self.font_desc.set_size(pango_size * scale / 100)
         self.__font_name = font_name
         self.layout.set_font_description(self.font_desc)
         self.layout.set_wrap(pango.WRAP_CHAR) # XXX
         # font for sstp message
         if self.side == 0:
-            self.sstp_font_desc = pango.FontDescription(self.__font_name)
+            self.sstp_font_desc = pango.FontDescription(font_name)
             pango_size = self.sstp_font_desc.get_size()
             if pango_size == 0:
                 default_size = 10 # for Windows environment
@@ -491,20 +466,12 @@ class BalloonWindow:
             if self.__shown:
                 self.darea.queue_draw()
 
-    def set_alpha_channel(self, value):
-        self.__alpha_channel = value
-        self.reset_balloon()
-
-    def get_scale(self):
-        return self.__scale if self.__scalling else 100 # %
-
-    def set_scale(self, scale):
-        self.__scale = scale # %
-        self.reset_balloon()
-
-    def set_scalling(self, flag):
-        self.__scalling = flag
-        self.reset_balloon()
+    @property
+    def alpha_channel(self):
+        alpha = self.request_parent('GET', 'get_preference', 'balloon_alpha')
+        if alpha is None or not 0.1 <= alpha <= 1.0:
+            alpha = 1.0
+        return alpha
 
     def reset_sstp_marker(self):
         if self.side == 0:
@@ -615,10 +582,10 @@ class BalloonWindow:
 
     def set_balloon(self, num):
         self.num = num
-        balloon_id = self.id_format % (num * 2 + self.direction)
+        balloon_id = self.id_format.format(num * 2 + self.direction)
         self.balloon_pixbuf = self.get_pixbuf(balloon_id)
         if self.balloon_pixbuf is None:
-            balloon_id = self.id_format % (0 + self.direction)
+            balloon_id = self.id_format.format(0 + self.direction)
             self.balloon_pixbuf = self.get_pixbuf(balloon_id)
         assert self.balloon_pixbuf is not None
         self.balloon_id = balloon_id
@@ -628,16 +595,13 @@ class BalloonWindow:
         self.darea.set_size_request(self.width, self.height)
         mask = gtk.gdk.Pixmap(None, self.width, self.height, 1)
         self.balloon_pixbuf[0].render_threshold_alpha(
-            mask, 0, 0, 0, 0, self.width, self.height, 255)
-        if self.window.is_composited():
-            self.window.input_shape_combine_mask(mask, 0, 0)
-        else:
-            self.window.shape_combine_mask(mask, 0, 0)
+            mask, 0, 0, 0, 0, self.width, self.height, 1)
+        self.window.mask = mask
         self.reset_arrow()
         self.reset_sstp_marker()
         self.reset_message_regions()
         if self.__shown:
-            self.callback['position_balloons']()
+            self.request_parent('NOTIFY', 'position_balloons')
             self.darea.queue_draw()
 
     def set_direction(self, dir):
@@ -654,7 +618,7 @@ class BalloonWindow:
             value = default_value
         if value < 0:
             value = base + value
-        return int(value * self.get_scale() / 100)
+        return int(value * self.scale / 100)
 
     def __get_with_scaling(self, name, conv, default_value):
         path, config = self.balloon[self.balloon_id]
@@ -663,13 +627,22 @@ class BalloonWindow:
             value = self.desc.get_with_type(name, conv)
             if value is None:
                 value = default_value
-        return conv(value * self.get_scale() / 100)
+        return conv(value * self.scale / 100)
 
     def __move(self):
         x, y = self.get_position()
-        self.window.move(x, y)
+        self.window.resize_move(x, y)
 
-    def set_position(self, x, y):
+    def set_position(self, base_x, base_y):
+        if self.balloon_id is None: ## FIXME
+            return
+        px, py = self.get_balloon_windowposition()
+        if self.direction == 0:
+            w, h = self.get_balloon_size()
+            x = base_x + px - w
+        else:
+            x = base_x + px
+        y = base_y + py
         self.position = (x, y)
         if self.__shown:
             self.__move()
@@ -698,8 +671,7 @@ class BalloonWindow:
     def hide(self):
         if not self.__shown:
             return
-        ##self.window.hide()
-        self.darea.hide()
+        self.window.hide()
         self.__shown = 0
         self.images = []
 
@@ -714,16 +686,16 @@ class BalloonWindow:
     def show_sstp_message(self, message, sender):
         if self.sstp_region is None:
             self.show()
-        self.sstp_message = '%s (%s)' % (message, sender)
+        self.sstp_message = '{0} ({1})'.format(message, sender)
         x, y, w, h = self.sstp_region
         self.sstp_layout.set_text(self.sstp_message)
         message_width, message_height = self.sstp_layout.get_pixel_size()
         if message_width > w:
-            self.sstp_message = u'... (%s)' % sender
+            self.sstp_message = u'... ({0})'.format(sender)
             i = 0
             while 1:
                 i += 1
-                s = '%s... (%s)' % (message[:i], sender)
+                s = '{0}... ({1})'.format(message[:i], sender)
                 self.sstp_layout.set_text(s)
                 message_width, message_height = \
                                self.sstp_layout.get_pixel_size()
@@ -815,11 +787,11 @@ class BalloonWindow:
         cr.paint()
         cr.restore()
         cr.set_source_pixbuf(self.balloon_pixbuf[0], 0, 0)
-        cr.paint_with_alpha(self.__alpha_channel)
+        cr.paint_with_alpha(self.alpha_channel)
         # draw images
         for i in range(len(self.images)):
             pixbuf, (w, h), (x, y) = self.images[i]
-            scale = self.get_scale()
+            scale = self.scale
             w = max(8, int(w * scale / 100))
             h = max(8, int(h * scale / 100))
             if x == 'centerx':
@@ -840,7 +812,7 @@ class BalloonWindow:
                     continue
             pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             cr.set_source_pixbuf(pixbuf, x, y)
-            cr.paint_with_alpha(self.__alpha_channel)
+            cr.paint_with_alpha(self.alpha_channel)
         # draw text
         i = self.lineno
         j = len(self.text_buffer)
@@ -851,18 +823,15 @@ class BalloonWindow:
             x, y, w, h = self.line_regions[line]
             if self.text_buffer[i].endswith('\n[half]'):
                 new_y = int(y + (self.font_height + self.line_space) / 2)
-                self.update_line_regions(line + 1, new_y)
                 markup = self.set_markup(i, self.text_buffer[i][:-7])
-                self.layout.set_markup(markup)
-                cr.set_source_rgba(*self.text_normal_color)
-                cr.move_to(x, y)
-                cr.show_layout(self.layout)
             else:
+                new_y = int(y + self.font_height + self.line_space)
                 markup = self.set_markup(i, self.text_buffer[i])
-                self.layout.set_markup(markup)
-                cr.set_source_rgba(*self.text_normal_color)
-                cr.move_to(x, y)
-                cr.show_layout(self.layout)
+            self.update_line_regions(line + 1, new_y)
+            self.layout.set_markup(markup)
+            cr.set_source_rgba(*self.text_normal_color)
+            cr.move_to(x, y)
+            cr.show_layout(self.layout)
             if self.sstp_pixbuf is not None:
                 for l, c in self.sstp_marker:
                     if l == i:
@@ -873,7 +842,7 @@ class BalloonWindow:
                         my = y + (self.font_height + self.line_space) / 2
                         my = my - mh / 2
                         cr.set_source_pixbuf(self.sstp_pixbuf[0], mx, my)
-                        cr.paint_with_alpha(self.__alpha_channel)
+                        cr.paint_with_alpha(self.alpha_channel)
             i += 1
             line += 1
         del cr
@@ -975,11 +944,12 @@ class BalloonWindow:
             if self.selection != new_selection:
                 sl, sn, el, en, link_id, raw_text, text = \
                     self.link_buffer[new_selection]
-                self.callback['notify_event'](
+                self.request_parent(
+                    'NOTIFY', 'notify_event',
                     'OnChoiceEnter', raw_text, link_id, self.selection)
         else:
             if self.selection is not None:
-                self.callback['notify_event']('OnChoiceEnter')
+                self.request_parent('NOTIFY', 'notify_event', 'OnChoiceEnter')
         if new_selection == self.selection:
             return 0
         else:
@@ -1016,10 +986,11 @@ class BalloonWindow:
         return True
 
     def button_press(self, darea, event):
-        self.callback['reset_idle_time']()
+        self.request_parent('NOTIFY', 'reset_idle_time')
         click = 1 if event.type == gtk.gdk.BUTTON_PRESS else 2
-        if self.callback['is_paused']():
-            self.callback['notify_balloon_click'](event.button, click, self.side)
+        if self.request_parent('GET', 'is_paused'):
+            self.request_parent('NOTIFY', 'notify_balloon_click',
+                                event.button, click, self.side)
             return True
         # arrows
         px = int(event.x)
@@ -1045,11 +1016,12 @@ class BalloonWindow:
         if self.selection is not None:
             sl, sn, el, en, link_id, raw_text, text = \
                 self.link_buffer[self.selection]
-            self.callback['notify_link_selection'](
-                link_id, raw_text, self.selection)
+            self.request_parent('NOTIFY', 'notify_link_selection',
+                                link_id, raw_text, self.selection)
             return True
         # balloon's background
-        self.callback['notify_balloon_click'](event.button, click, self.side)
+        self.request_parent('NOTIFY', 'notify_balloon_click',
+                            event.button, click, self.side)
         return True
 
     def clear_text(self):
@@ -1227,16 +1199,18 @@ class BalloonWindow:
                 self.darea.queue_draw()
 
 
-class CommunicateWindow:
+class CommunicateWindow(object):
 
     NAME = ''
     ENTRY = ''
 
-    def __init__(self, callback, debug):
-        self.callback = callback
-        self.debug = debug
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.window = None
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def new(self, desc, balloon):
         if self.window:
             self.window.destroy()
@@ -1356,14 +1330,14 @@ class CommunicateBox(CommunicateWindow):
     def delete(self, widget, event):
         self.window.hide()
         self.cancel()
-        self.callback['reset_user_interaction']()
+        self.request_parent('NOTIFY', 'reset_user_interaction')
         return True
 
     def key_press(self, widget, event):
         if event.keyval == gtk.keysyms.Escape:
             self.window.hide()
             self.cancel()
-            self.callback['reset_user_interaction']()
+            self.request_parent('NOTIFY', 'reset_user_interaction')
             return True
         return False
 
@@ -1382,7 +1356,8 @@ class CommunicateBox(CommunicateWindow):
         if data:
             data = unicode(data, 'utf-8')
         if data is not None:
-            self.callback['notify_event']('OnCommunicate', 'user', data)
+            self.request_parent('NOTIFY', 'notify_event',
+                                'OnCommunicate', 'user', data)
 
 
 class TeachBox(CommunicateWindow):
@@ -1399,8 +1374,8 @@ class TeachBox(CommunicateWindow):
     def send(self, data):
         if data:
             data = unicode(data, 'utf-8')
-        self.callback['notify_user_teach'](data)
-        self.callback['reset_user_interaction']()
+        self.request_parent('NOTIFT', 'notify_user_teach', data)
+        self.request_parent('NOTIFY', 'reset_user_interaction')
 
 
 class InputBox(CommunicateWindow):
@@ -1456,13 +1431,13 @@ class InputBox(CommunicateWindow):
             data = ''
         ## CHECK: symbol
         if self.symbol == 'OnUserInput' and \
-           self.callback['notify_event']('OnUserInput', data):
+           self.request_parent('GET', 'notify_event', 'OnUserInput', data):
             pass
-        elif self.callback['notify_event']('OnUserInput', self.symbol, data):
+        elif self.request_parent('GET', 'notify_event', 'OnUserInput', self.symbol, data):
             pass
-        elif self.callback['notify_event'](self.symbol, data):
+        elif self.request_parent('GET', 'notify_event', self.symbol, data):
             pass
-        self.callback['reset_user_interaction']()
+        self.request_parent('NOTIFY', 'reset_user_interaction')
 
 
 def test():
