@@ -14,12 +14,17 @@
 
 import sys
 import imp
+import logging
 import os
-import StringIO
 import codecs
 
 
-class SAORI:
+def get_path(): # XXX
+    import ninix
+    return os.path.join(ninix.__path__[0], 'dll')
+
+
+class SAORI(object):
 
     RESPONSE = {204: 'SAORI/1.0 204 No Content\r\n\r\n',
                 400: 'SAORI/1.0 400 Bad Request\r\n\r\n',
@@ -77,8 +82,7 @@ class SAORI:
         req_type = None
         argument = []
         self.charset = 'Shift_JIS' # default
-        header = StringIO.StringIO(req)
-        for line in header:
+        for line in req.splitlines():
             line = line.strip()
             if not line:
                 continue
@@ -89,29 +93,29 @@ class SAORI:
                 continue
             if ':' not in line:
                 continue
-            key, value = [x.strip() for x in line.split(':', 1)]
+            key, value = line.split(':', 1)
+            key = key.strip()
             if key == 'Charset':
-                charset = value
+                charset = value.strip()
                 try:
                     codecs.lookup(charset)
                 except:
-                    sys.stderr.write('Unsupported charset %s' % repr(charset))
+                    logging.warning(
+                        'Unsupported charset {0}'.format(repr(charset)))
                 else:
                     self.charset = charset
             if key.startswith('Argument'): ## FIXME
                 argument.append(value)
             else:
                 continue
-        for i in range(len(argument)):
-            argument[i] = unicode(argument[i], self.charset, 'ignore')
+        argument = [unicode(value, self.charset, 'ignore').strip() for value in argument]
         return req_type, argument
 
-class Library:
 
-    def __init__(self, dll_type, dir, sakura=None, saori_lib=None):
+class Library(object):
+
+    def __init__(self, dll_type, sakura=None, saori_lib=None):
         self.__type = dll_type
-        self.__dir_list = []
-        self.__dir_list.append(dir)
         self.__sakura = sakura
         self.__saori_lib = saori_lib
 
@@ -123,36 +127,35 @@ class Library:
         name = name.replace('\\', '/')
         head, tail = os.path.split(name)
         name = tail
-        if name:
-            if name.endswith('.dll'):
-                name = name[:-4]
-        else:
+        if not name:
             return None
+        if name.lower().endswith('.dll'): # XXX
+            name = name[:-4]
         module = self.__import_module(name)
         if not module:
             return None
-        else:
-            instance = None
-            if self.__type == 'saori':
-                if getattr(module, 'Saori', None):
-                    saori = module.Saori()
-                    if getattr(saori, 'need_ghost_backdoor', None):
-                        saori.need_ghost_backdoor(self.__sakura)
-                else:
-                    saori = None
-                instance = saori
-            elif self.__type == 'shiori':
-                if getattr(module, 'Shiori', None):
-                    shiori = module.Shiori(dll_name)
-                    if getattr(shiori, 'use_saori', None):
-                        shiori.use_saori(self.__saori_lib)
-                else:
-                    shiori = None
-                instance = shiori
-            if instance is None:
-                del module
-                del sys.modules[name]
-            return instance
+        instance = None
+        if self.__type == 'saori':
+            if getattr(module, 'Saori', None):
+                saori = module.Saori()
+                if getattr(saori, 'need_ghost_backdoor', None):
+                    saori.need_ghost_backdoor(self.__sakura)
+            else:
+                saori = None
+            instance = saori
+        elif self.__type == 'shiori':
+            if getattr(module, 'Shiori', None):
+                shiori = module.Shiori(dll_name)
+                if getattr(shiori, 'use_saori', None):
+                    shiori.use_saori(self.__saori_lib)
+            else:
+                shiori = None
+            instance = shiori
+        if instance is None:
+            del module
+            ## this is NOT proper: infects the working ghost(s).
+            ##del sys.modules[name]
+        return instance
 
     def __import_module(self, name):
         fp = None
@@ -161,7 +164,8 @@ class Library:
         except:
             pass
         try:
-            fp, pathname, description = imp.find_module(name, self.__dir_list)
+            path = get_path()
+            fp, pathname, description = imp.find_module(name, [path])
         except:
             return None
         try:
