@@ -17,12 +17,12 @@
 import random
 import time
 import sys
+import logging
 import os
 import re
-import StringIO
 
 
-REVISION = '$Revision: 1.28 $'
+REVISION = '$Revision: 1.28 $' ## FIXME
 
 # tree node types
 ADD_EXPR     = 1
@@ -43,10 +43,9 @@ def list_dict(top_dir):
     return buf
 
 
-class NiseShiori:
+class NiseShiori(object):
 
-    def __init__(self, debug=0):
-        self.debug = debug
+    def __init__(self):
         self.dict = {}
         self.type_chains = {}
         self.word_chains = {}
@@ -88,42 +87,42 @@ class NiseShiori:
 
     def load_database(self, path):
         try:
-            f = open(path)
+            with open(path) as f:
+                dic = {}
+                for line in f:
+                    if line.startswith('# ns_st: '):
+                        try:
+                            self.ai_talk_interval = int(line[9:])
+                        except ValueError:
+                            pass
+                        continue
+                    elif line.startswith('# ns_tn: '):
+                        self.username = line[9:].strip()
+                        continue
+                    try:
+                        name, value = [s.strip() for s in line.split('=')]
+                    except ValueError:
+                        logging.error(
+                            'niseshiori.py: malformed database (ignored)')
+                        return
+                    dic[name] = value
         except IOError:
             return
-        dic = {}
-        for line in f:
-            if line.startswith('# ns_st: '):
-                try:
-                    self.ai_talk_interval = int(line[9:])
-                except ValueError:
-                    pass
-                continue
-            elif line.startswith('# ns_tn: '):
-                self.username = line[9:].strip()
-                continue
-            try:
-                name, value = [s.strip() for s in line.split('=')]
-            except ValueError:
-                print 'niseshiori.py: malformed database (ignored)'
-                return
-            dic[name] = value
         self.variables = dic
 
     def save_database(self):
         if self.dbpath is None:
             return
         try:
-            f = open(self.dbpath, 'w')
+            with open(self.dbpath, 'w') as f:
+                f.write('# ns_st: {0:d}\n'.format(self.ai_talk_interval))
+                f.write('# ns_tn: {0}\n'.format(self.username))
+                for name, value in self.variables.items():
+                    if not name.startswith('_'):
+                        f.write('{0}={1}\n'.format(name, str(value)))
         except IOError:
-            print 'niseshiori.py: cannot write database (ignored)'
+            logging.error('niseshiori.py: cannot write database (ignored)')
             return
-        f.write('# ns_st: %d\n' % self.ai_talk_interval)
-        f.write('# ns_tn: %s\n' % self.username)
-        for name, value in self.variables.iteritems():
-            if not name.startswith('_'):
-                f.write('%s=%s\n' % (name, str(value)))
-        f.close()
 
     def finalize(self):
         pass
@@ -134,12 +133,13 @@ class NiseShiori:
 
     def read_dict(self, path):
         # read dict file and decrypt if necessary
-        f = open(path)
-        if path.endswith('.dtx'):
-            buf = self.decrypt(f.read())
-        else:
-            buf = f.readlines()
-        f.close()
+        with open(path) as f:
+            basename, ext = os.path.splitext(path)
+            ext = ext.lower()
+            if ext == '.dtx':
+                buf = self.decrypt(f.read())
+            else:
+                buf = f.readlines()
         # omit empty lines and comments and merge continuous lines
         definitions = []
         decode = lambda line: unicode(line, 'Shift_JIS', 'replace')
@@ -330,9 +330,9 @@ class NiseShiori:
         return [s.replace('\\,', ',') for s in  buf]
 
     def syntax_error(self, path, line):
-        if self.debug & 4:
-            print 'niseshiori.py: syntax error in', os.path.basename(path)
-            print line
+        logging.debug(
+            'niseshiori.py: syntax error in {0}'.format(os.path.basename(path)))
+        logging.debug(line)
 
     re_comp_op = re.compile('<[>=]?|>=?|=')
     COND_COMPARISON = 1
@@ -510,7 +510,7 @@ class NiseShiori:
         self.event = event
         self.reference = tuple(ref)
         self.current_time = time.localtime(time.time())
-        for condition, actions in self.events.iteritems():
+        for condition, actions in self.events.items():
             if self.eval_condition(condition):
                 self.dict[key].extend(actions)
         script = self.get(key, default=None)
@@ -621,9 +621,8 @@ class NiseShiori:
         elif tag.startswith(r'\set[') and tag.endswith(']'):
             statement = tag[5:-1]
             if '=' not in statement:
-                if self.debug & 1024:
-                    print 'niseshiori.py: syntax error'
-                    print tag
+                logging.debug('niseshiori.py: syntax error')
+                logging.debug(tag)
             else:
                 name, expr = [x.strip() for x in statement.split('=', 1)]
                 value = self.eval_expression(expr)
@@ -631,11 +630,10 @@ class NiseShiori:
                     self.variables[name] = value
                     if not name.startswith('_'):
                         self.save_database()
-                    if self.debug & 2048:
-                        print 'set %s = "%s"' % (name, value)
-                elif self.debug & 1024:
-                    print 'niseshiori.py: syntax error'
-                    print tag
+                    logging.debug('set {0} = "{1}"'.format(name, value))
+                else:
+                    logging.debug('niseshiori.py: syntax error')
+                    logging.debug(tag)
         elif tag == r'\ns_rt':
             value = r'\a'
         elif tag == r'\ns_tn':
@@ -673,8 +671,7 @@ class NiseShiori:
                                 candidates_B.remove(word)
                             if not candidates_A and not candidates_B:
                                 del chains[chained_meta]
-                            if self.debug & 16:
-                                print 'chained: %s => %s' % (meta, word)
+                            logging.debug('chained: {0} => {1}'.format(meta, word))
                             break
                     else:
                         if match.group(4) == 'm?':
@@ -687,15 +684,14 @@ class NiseShiori:
                             word = self.expand(
                                 ''.join(('\\', match.group(4))), s)
                     chains = self.find_chains((chained_meta, word), s)
-                    if self.debug & 16:
-                        prefix = 'chain:'
-                        for k, (candidates_A, candidates_B) in chains.iteritems():
-                            for w in candidates_A:
-                                print prefix, '%s, %s' % (k, w)
-                                prefix = '      '
-                            for w in candidates_B:
-                                print prefix, '%s, %s' % (k, w)
-                                prefix = '      '
+                    prefix = 'chain:'
+                    for k, (candidates_A, candidates_B) in chains.items():
+                        for w in candidates_A:
+                            logging.debug('{0} {1}, {2}'.format(prefix, k, w))
+                            prefix = '      '
+                        for w in candidates_B:
+                            logging.debug('{0} {1}, {2}'.format(prefix, k, w))
+                            prefix = '      '
                     variables[meta] = word
                     variable_chains.append(chains)
                 buf.append(variables[meta])
@@ -709,8 +705,7 @@ class NiseShiori:
         choices = []
         for keyword, word in self.type_chains.get(key, []):
             if keyword in context:
-                if self.debug & 16:
-                    print 'chain keyword:', keyword
+                logging.debug('chain keyword: {0}'.format(keyword))
                 choices.append(word)
         if not choices:
             match = self.re_category.match(key)
@@ -718,20 +713,19 @@ class NiseShiori:
                 key = match.groups()
             choices = self.dict.get(key)
         if not choices:
-            if self.debug & 8:
-                if isinstance(key, tuple):
-                    key = '(%s, %s)' % key
-                sys.stderr.write('%s not found\n' % key)
+            if isinstance(key, tuple):
+                key = '({0}, {1})'.format(*key)
+            logging.debug('{0} not found'.format(key))
             return default
         s = random.choice(choices)
         t = self.replace_meta(s)
-        if self.debug & 16:
-            if isinstance(key, tuple):
-                key = '\\%s[%s]' % (key[0] or '', key[1])
-            print key.encode('utf-8', 'ignore'), '=>', \
-                  s.encode('utf-8', 'ignore')
-            print ' ' * len(key),
-            print '=>', t.encode('utf-8', 'ignore')
+        if isinstance(key, tuple):
+            key = '\\{0}[{1}]'.format(key[0] or '', key[1])
+        logging.debug(
+            ''.join((key.encode('utf-8', 'ignore'), '=>',
+                     s.encode('utf-8', 'ignore'))))
+        logging.debug(
+            ''.join((' ' * len(key), '=>', t.encode('utf-8', 'ignore'))))
         return t
 
     def find_chains(self, key, context):
@@ -798,7 +792,7 @@ class NiseShiori:
             result = str(self.surf0)
         elif name == '%surf1':
             result = str(self.surf1)
-        elif name in ['%plathome', '%platform']:
+        elif name in ['%plathome', '%platform']: ## FIXME
             result = 'ninix'
         elif name == '%move':
             result = str(self.motion_count)
@@ -806,9 +800,9 @@ class NiseShiori:
             result = str(self.mikire)
         elif name == '%kasanari':
             result = str(self.kasanari)
-        elif name == '%ver':
+        elif name == '%ver': ## FIXME
             if REVISION[1:11] == 'Revision: ':
-                result = '偽栞 for ninix (rev.%s)' % REVISION[11:-2]
+                result = '偽栞 for ninix (rev.{0})'.format(REVISION[11:-2])
             else:
                 result = '偽栞 for ninix'
         elif name == '%sender':
@@ -891,23 +885,23 @@ class NiseShiori:
             raise RuntimeError, 'should not reach here'
 
     def is_number(self, s):
-        return s and filter(lambda c: c in '0123456789', s) == s
+        return s and all([bool(c in '0123456789') for c in s])
+
 
 class ExprError(ValueError):
     pass
 
-class ExprParser:
+
+class ExprParser(object):
 
     def __init__(self):
-        self.debug = 0
-        # bit 0 = trace get_expr()
-        # bit 1 = trace all 'get_' functions
+        pass
 
     def show_progress(self, func, buf):
         if buf is None:
-            print '%s() -> syntax error' % func
+            logging.debug('{0}() -> syntax error'.format(func))
         else:
-            print '%s() -> %s' % (func, buf)
+            logging.debug('{0}() -> {1}'.format(func, buf))
 
     re_token = re.compile(r'[()*/\+-]|\d+|\s+')
 
@@ -969,8 +963,7 @@ class ExprParser:
         buf = self.get_add_expr()
         if not self.done():
             raise ExprError
-        if self.debug & 3:
-            self.show_progress('get_expr', buf)
+        self.show_progress('get_expr', buf)
         return buf
 
     def get_add_expr(self):
@@ -982,8 +975,7 @@ class ExprParser:
             buf.append(self.pop()) # operator
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_add_expr', buf)
+        self.show_progress('get_add_expr', buf)
         return buf
 
     def get_mul_expr(self):
@@ -995,8 +987,7 @@ class ExprParser:
             buf.append(self.pop()) # operator
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_mul_expr', buf)
+        self.show_progress('get_mul_expr', buf)
         return buf
 
     def get_unary_expr(self):
@@ -1004,8 +995,7 @@ class ExprParser:
             buf = [UNARY_EXPR, self.pop(), self.get_unary_expr()]
         else:
             buf = self.get_primary_expr()
-        if self.debug & 2:
-            self.show_progress('get_unary_expr', buf)
+        self.show_progress('get_unary_expr', buf)
         return buf
 
     def get_primary_expr(self):
@@ -1015,8 +1005,7 @@ class ExprParser:
             self.match(')')
         else:
             buf = [PRIMARY_EXPR, self.pop()]
-        if self.debug & 2:
-            self.show_progress('get_primary_expr', buf)
+        self.show_progress('get_primary_expr', buf)
         return buf
 
 # <<< EXPRESSION SYNTAX >>>
@@ -1029,10 +1018,11 @@ class ExprParser:
 # unary-op     := '+' | '-'
 # primary-expr := identifier | constant | '(' add-expr ')'
 
+
 class Shiori(NiseShiori):
 
-    def __init__(self, dll_name, debug=0):
-        NiseShiori.__init__(self, debug)
+    def __init__(self, dll_name):
+        NiseShiori.__init__(self)
         self.dll_name = dll_name
 
     def load(self, top_dir):
@@ -1049,17 +1039,17 @@ class Shiori(NiseShiori):
         return result
 
     def show_description(self):
-        sys.stdout.write(
+        logging.info(
             'Shiori: NiseShiori compatible module for ninix\n'
             '        Copyright (C) 2001, 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
             '        Copyright (C) 2002-2011 by Shyouzou Sugitani\n'
-            '        Copyright (C) 2003 by Shun-ichi TAHARA\n')
+            '        Copyright (C) 2003 by Shun-ichi TAHARA')
 
     def request(self, req_string):
-        header = StringIO.StringIO(req_string)
+        header = req_string.splitlines()
         req_header = {}
-        line = header.readline()
+        line = header.pop(0)
         if line:
             line = line.strip()
             req_list = line.split()
@@ -1081,9 +1071,7 @@ class Shiori(NiseShiori):
         result = ''
         to = None
         if 'ID' in req_header:
-            if req_header['ID'] == 'charset':
-                result = 'UTF-8'
-            elif req_header['ID'] == 'dms':
+            if req_header['ID'] == 'dms':
                 result = self.getdms()
             elif req_header['ID'] == 'OnAITalk':
                 result = self.getaistringrandom()
@@ -1122,15 +1110,16 @@ class Shiori(NiseShiori):
             to = self.communicate_to()
         result = 'SHIORI/3.0 200 OK\r\n' \
                  'Sender: Niseshiori\r\n' \
-                 'Value: %s\r\n' % result
+                 'Charset: UTF-8\r\n' \
+                 'Value: {0}\r\n'.format(result)
         if to is not None:
-            result = ''.join((result, 'Reference0: %s\r\n' % to))
+            result = ''.join((result, 'Reference0: {0}\r\n'.format(to)))
         result = ''.join((result, '\r\n'))
         return result
 
 
-def ns_open(shiori_dir, debug=0):
-    ns = NiseShiori(debug)
+def ns_open(shiori_dir):
+    ns = NiseShiori()
     ns.load(shiori_dir)
     return ns
 
@@ -1153,42 +1142,42 @@ def test():
 
 def dump_dict(ns):
     print 'DICT'
-    for k, v in ns.dict.iteritems():
+    for k, v in ns.dict.items():
         if isinstance(k, tuple):
-            k = '(%s, %s)' % k
+            k = '({0}, {1})'.format(*k)
         print k.encode('utf-8', 'ignore')
         for e in v: print '\t', e.encode('utf-8', 'ignore')
     print '*' * 50
     print 'CHAINS'
-    for t, chain_list in ns.type_chains.iteritems():
+    for t, chain_list in ns.type_chains.items():
         print t.encode('utf-8', 'ignore')
         for c, w in chain_list:
-            print ('-> %s, %s' % (c, w)).encode('utf-8', 'ignore')
-    for key, dic in ns.word_chains.iteritems():
-        print ('(%s, %s)' % key).encode('utf-8', 'ignore')
-        for t, chain_list in dic.iteritems():
-            prefix = '-> %s' % t.encode('utf-8', 'ignore')
+            print ('-> {0}, {1}'.format(c, w)).encode('utf-8', 'ignore')
+    for key, dic in ns.word_chains.items():
+        print ('({0}, {1})'.format(*key)).encode('utf-8', 'ignore')
+        for t, chain_list in dic.items():
+            prefix = '-> {0}'.format(t.encode('utf-8', 'ignore'))
             for c, w in chain_list:
-                print prefix, ('-> %s, %s' % (c, w)).encode('utf-8', 'ignore')
+                print prefix, ('-> {0}, {1}'.format(c, w)).encode('utf-8', 'ignore')
                 prefix = ''.join(('   ', ' ' * len(t)))
     print '*' * 50
     print 'KEYWORDS'
-    for (t, w), s in ns.keywords.iteritems():
+    for (t, w), s in ns.keywords.items():
         print t.encode('utf-8', 'ignore'), w.encode('utf-8', 'ignore')
         print '->', s.encode('utf-8', 'ignore')
     print '*' * 50
     print 'RESPONSES'
-    for k, v in ns.responses.iteritems():
+    for k, v in ns.responses.items():
         print_condition(k)
         print '->', v
     print '*' * 50
     print 'GREETINGS'
-    for k, v in ns.greetings.iteritems():
+    for k, v in ns.greetings.items():
         print k.encode('utf-8', 'ignore')
         print '->', v
     print '*' * 50
     print 'EVENTS'
-    for k, v in ns.events.iteritems():
+    for k, v in ns.events.items():
         print_condition(k)
         for e in v:
             print '->', e.encode('utf-8', 'ignore')
@@ -1197,9 +1186,9 @@ def print_condition(condition):
     prefix = 'Condition'
     for cond_type, expr in condition:
         if cond_type == NiseShiori.COND_COMPARISON:
-            print prefix, ("'%s' '%s' '%s'" % expr).encode('utf-8', 'ignore')
+            print prefix, ("'{0}' '{1}' '{2}'".format(*expr)).encode('utf-8', 'ignore')
         elif cond_type == NiseShiori.COND_STRING:
-            print prefix, "'%s'" % expr.encode('utf-8', 'ignore')
+            print prefix, "'{0}'".format(expr.encode('utf-8', 'ignore'))
         prefix = '      and'
 
 if __name__ == '__main__':
