@@ -25,13 +25,14 @@
 # - ＄BalloonOffset0, ＄BalloonOffset1
 # - マルチキャラクタ
 
-import __builtin__
 import os
 import sys
+import logging
 import re
 import time
 import random
-import StringIO
+
+from ninix.home import get_normalized_path
 
 NODE_TEXT       = 1
 NODE_REF        = 2
@@ -80,15 +81,17 @@ def list_dict(top_dir):
     except OSError:
         dir_list = []
     for filename in dir_list:
-        if filename.startswith('dic') and \
-           (filename.endswith('.txt') or filename.endswith('.sat')) or \
-           filename in ['replace.txt', 'replace_after.txt',
-                        'satori_conf.txt', 'satori_conf.sat']:
+        basename, ext = os.path.splitext(filename)
+        ext = ext.lower()
+        if filename.lower().startswith('dic') and \
+           ext in ['.txt', '.sat'] or \
+           filename.lower() in ['replace.txt', 'replace_after.txt',
+                                'satori_conf.txt', 'satori_conf.sat']: # XXX
             buf.append(os.path.join(top_dir, filename))
     return buf
 
 
-class Filter:
+class Filter(object):
 
     def __init__(self, rules):
         self.rules = []
@@ -103,40 +106,35 @@ class Filter:
 
 ###   PARSER   ###
 
-def read_tab_file(path, encrypted=0, debug=0):
-    f = __builtin__.open(path)
+def read_tab_file(path, encrypted=0):
     lineno = 0
     buf = []
-    for line in f:
-        lineno += 1
-        if line.endswith('\r\n'):
-            line = line[:-2]
-        elif line.endswith('\r') or line.endswith('\n'):
-            line = line[:-1]
-        if encrypted:
-            line = ''.join(decrypt(decrypt(line)))
-        try:
-            line = unicode(line, 'Shift_JIS')
-        except UnicodeError, e:
-            if debug & 4:
-                print 'satori.py: %s in %s (line %d)' % (e, path, lineno)
-            continue
-        try:
-            old, new = line.split('\t')
-        except ValueError:
-            if debug & 4:
-                print 'satori.py: invalid line in %s (line %d)' % \
-                      (path, lineno)
-            continue
-        buf.append((old, new))
-    f.close()
+    with open(path) as f:
+        for line in f:
+            lineno += 1
+            if line.endswith('\r\n'):
+                line = line[:-2]
+            elif line.endswith('\r') or line.endswith('\n'):
+                line = line[:-1]
+            if encrypted:
+                line = ''.join(decrypt(decrypt(line)))
+            try:
+                line = unicode(line, 'Shift_JIS')
+            except UnicodeError as e:
+                logging.debug('satori.py: {0} in {1} (line {2:d})'.format(e, path, lineno))
+                continue
+            try:
+                old, new = line.split('\t')
+            except ValueError:
+                logging.debug('satori.py: invalid line in {0} (line {1:d})'.format(path, lineno))
+                continue
+            buf.append((old, new))
     return buf
 
 
-class Parser:
+class Parser(object):
 
-    def __init__(self, debug=0):
-        self.debug = debug
+    def __init__(self):
         self.talk = {}
         self.word = {}
         self.variable = []
@@ -169,24 +167,25 @@ class Parser:
             return 0
 
     def load_replace_file(self, path):
-        self.replace_filter = Filter(read_tab_file(path, debug=self.debug))
+        self.replace_filter = Filter(read_tab_file(path))
 
     def get_dict(self):
         return self.talk, self.word
 
     def read(self, path):
-        if path.endswith('.sat'):
+        basename, ext = os.path.splitext(path)
+        ext = ext.lower()
+        if ext == '.sat':
             encrypted = 1
         else:
             encrypted = 0
         filename = os.path.basename(path)
-        if filename.startswith('dicanchor'):
+        if filename.lower().startswith('dicanchor'): # XXX
             self.is_anchor = 1
         else:
             self.is_anchor = 0
-        f = __builtin__.open(path)
-        self.read_file(f, path, encrypted)
-        f.close()
+        with open(path) as f:
+            self.read_file(f, path, encrypted)
         if self.is_anchor:
             self.anchor_filter = Filter(self.anchor_list)
 
@@ -201,6 +200,7 @@ class Parser:
             else:
                 line = ''.join((line_buffer, line))
                 line_buffer = None
+            phi_escape[lineno] = []
             if line.endswith('\r\n'):
                 line = line[:-2]
             elif line.endswith('\r') or line.endswith('\n'):
@@ -209,16 +209,13 @@ class Parser:
                 line = ''.join(decrypt(decrypt(line)))
             try:
                 line = unicode(line, 'Shift_JIS')
-            except UnicodeError, e:
-                if self.debug & 4:
-                    if path is None:
-                        print 'satori.py: %s (line %d)' % (e, lineno)
-                    else:
-                        print 'satori.py: %s in %s (line %d)' % \
-                              (e, path, lineno)
+            except UnicodeError as e:
+                if path is None:
+                    logging.debug('satori.py: {0} (line {1:d})'.format(e, lineno))
+                else:
+                    logging.debug('satori.py: {0} in {1} (line {2:d})'.format(e, path, lineno))
                 continue
             pos = 0
-            phi_escape[lineno] = []
             while line.count(u'φ', pos) > 0:
                 pos = line.find(u'φ', pos)
                 if pos == len(line) - 1:
@@ -259,7 +256,7 @@ class Parser:
         self.count['Line'] = self.count['Line'] + lineno
         talk = 0
         eventtalk = 0
-        for key, value in self.talk.iteritems():
+        for key, value in self.talk.items():
             number = len(value)
             talk += number
             if key.startswith('On'):
@@ -273,7 +270,7 @@ class Parser:
                                   - self.count['EventTalk']
         self.count['Words'] = len(self.word)
         word = 0
-        for value in self.word.itervalues():
+        for value in self.word.values():
             word += len(value)
         self.count['Word'] = word
         self.count['Anchor'] = len(self.anchor_list)
@@ -298,8 +295,8 @@ class Parser:
             num_close += line.count(u'）') ### FIXME: φ
             if num_open > 0 and num_open != num_close:
                 if n == len(linelist) - 1:
-                    if self.debug & 4:
-                        print 'satori.py: syntax error (unbalanced parens)'
+                    logging.debug(
+                        'satori.py: syntax error (unbalanced parens)')
                 else:
                     prev = ''.join((prev, linelist[n]))
                     continue 
@@ -337,7 +334,7 @@ class Parser:
             talk_list.append(buf)
             if self.is_anchor:
                 self.anchor_list.append(
-                    [name, '\\_a[%s]%s\\_a' % (name, name)])
+                    [name, '\\_a[{0}]{0}\\_a'.format(name)])
 
     def parse_word_group(self, linelist, phi_escape):
         lineno = linelist[0]
@@ -354,8 +351,8 @@ class Parser:
             num_close += line.count(u'）') ### FIXME: φ
             if num_open > 0 and num_open != num_close:
                 if n == len(linelist) - 1:
-                    if self.debug & 4:
-                        print 'satori.py: syntax error (unbalanced parens)'
+                    logging.debug(
+                        'satori.py: syntax error (unbalanced parens)')
                 else:
                     prev = ''.join((prev, linelist[n]))
                     continue 
@@ -382,8 +379,7 @@ class Parser:
             if line[n] in ['\t', ' ', u'　', u'＝']: ### FIXME: φ
                 break
         else:
-            if self.debug & 4:
-                print 'satori.py: syntax error (expected a tab or equal)'
+            logging.debug('satori.py: syntax error (expected a tab or equal)')
             return None
         name_str = ''.join(line[1:n]) # XXX
         name = self.parse_word(line[1:n])
@@ -520,9 +516,12 @@ class Parser:
         assert line[0] == u'（'
         line = line[1:] ## FIXME
         depth = 1
+        count = 1
         while line:
             if line[0] == u'）': ### FIXME: φ
                 depth -= 1
+                if len(text_) != count: # （）
+                    text_.append('')
                 if depth == 0:
                     line = line[1:]
                     break
@@ -530,6 +529,7 @@ class Parser:
                 line = line[1:]
             elif line[0] == u'（': ### FIXME: φ
                 depth += 1
+                count += 1
                 text_.append(line[0])
                 line = line[1:]
             else:
@@ -587,7 +587,7 @@ class Parser:
         default = [[NODE_TEXT, line[:]]]
         try:
             line, buf = self.get_or_expr(line)
-        except ValueError, e:
+        except ValueError as e:
             return default
         if line:
             return default
@@ -747,7 +747,7 @@ class Parser:
         for node in node_list:
             indent = '  ' * depth
             if node[0] == NODE_TEXT:
-                print ''.join((indent, 'NODE_TEXT "%s"' % ''.join(node[1])))
+                print ''.join((indent, 'NODE_TEXT "{0}"'.format(''.join(node[1]))))
             elif node[0] == NODE_REF:
                 print ''.join((indent, 'NODE_REF'))
                 self.print_nodelist(node[1], depth + 1)
@@ -845,15 +845,14 @@ class Parser:
 
 ###   INTERPRETER   ###
 
-class Satori:
+class Satori(object):
 
     DBNAME = 'satori_savedata.txt'
     EDBNAME = 'satori_savedata.sat'
 
-    def __init__(self, satori_dir=os.curdir, debug=0):
+    def __init__(self, satori_dir=os.curdir):
         self.satori_dir = satori_dir
         self.dbpath = os.path.join(satori_dir, self.DBNAME)
-        self.debug = debug
         self.saori_function = {}
         self.parser = Parser()
         self.reset()
@@ -920,7 +919,7 @@ class Satori:
             self.expand(nodelist)
 
     def load_replace_file(self, path):
-        self.replace_filter = Filter(read_tab_file(path, debug=self.debug))
+        self.replace_filter = Filter(read_tab_file(path))
 
     def load_database(self):
         if  self.variable.get(u'セーブデータ暗号化') == u'有効':
@@ -930,7 +929,7 @@ class Satori:
             encrypted = 0
             self.dbpath = os.path.join(self.satori_dir, self.DBNAME)
         try:
-            database = read_tab_file(self.dbpath, encrypted, self.debug)
+            database = read_tab_file(self.dbpath, encrypted)
         except IOError:
             database = []
         for name, value in database:
@@ -944,64 +943,64 @@ class Satori:
             encrypted = 0
             self.dbpath = os.path.join(self.satori_dir, self.DBNAME)
         try:
-            f = __builtin__.open(self.dbpath, 'w')
+            with open(self.dbpath, 'w') as f:
+                for name, value in self.variable.items():
+                    if name in [u'前回終了時サーフェス0',
+                                u'前回終了時サーフェス1',
+                                u'デフォルトサーフェス0',
+                                u'デフォルトサーフェス1']:
+                        continue
+                    line = '{0}\t{1}'.format(name.encode('Shift_JIS'),
+                                             value.encode('Shift_JIS'))
+                    if encrypted:
+                        line = ''.join(encrypt(encrypt(line)))
+                    f.write(line)
+                    f.write('\r\n')
+                for side in [0, 1]:
+                    name = u'デフォルトサーフェス{0:d}'.format(side)
+                    value = self.to_zenkaku('{0:d}'.format(self.default_surface[side]))
+                    line = '{0}\t{1}'.format(name.encode('Shift_JIS'),
+                                             value.encode('Shift_JIS'))
+                    if encrypted:
+                        line = ''.join(encrypt(encrypt(line)))
+                    f.write(line)
+                    f.write('\r\n')
+                for side in [0, 1]:
+                    name = u'前回終了時サーフェス{0:d}'.format(side)
+                    value = self.to_zenkaku('{0:d}'.format(self.current_surface[side]))
+                    line = '{0}\t{1}'.format(name.encode('Shift_JIS'),
+                                             value.encode('Shift_JIS'))
+                    if encrypted:
+                        line = ''.join(encrypt(encrypt(line)))
+                    f.write(line)
+                    f.write('\r\n')
+                name = u'起動回数'
+                value = self.to_zenkaku('{0:d}'.format(self.runcount))
+                line = '{0}\t{1}'.format(name.encode('Shift_JIS'),
+                                         value.encode('Shift_JIS'))
+                if encrypted:
+                    line = ''.join(encrypt(encrypt(line)))
+                f.write(line)
+                f.write('\r\n')
+                for name in self.timer:
+                    value = self.to_zenkaku(self.timer[name])
+                    line = '{0}\t{1}'.format(name.encode('Shift_JIS'),
+                                             value.encode('Shift_JIS'))
+                    if encrypted:
+                        line = ''.join(encrypt(encrypt(line)))
+                    f.write(line)
+                    f.write('\r\n')
+                for name in self.reserved_talk:
+                    value = self.to_zenkaku(self.reserved_talk[name])
+                    line = '{0}\t{1}'.format(''.join((u'次から', value, u'回目のトーク')).encode('Shift_JIS'),
+                                             name.encode('Shift_JIS'))
+                    if encrypted:
+                        line = ''.join(encrypt(encrypt(line)))
+                    f.write(line)
+                    f.write('\r\n')
         except IOError:
-            if self.debug & 4:
-                print 'satori.py: cannot write', self.dbpath
+            logging.debug('satori.py: cannot write {0}'.format(self.dbpath))
             return
-        for name, value in self.variable.iteritems():
-            if name in [u'前回終了時サーフェス0', u'前回終了時サーフェス1',
-                        u'デフォルトサーフェス0', u'デフォルトサーフェス1']:
-                continue
-            line = '%s\t%s' % (name.encode('Shift_JIS'),
-                               value.encode('Shift_JIS'))
-            if encrypted:
-                line = ''.join(encrypt(encrypt(line)))
-            f.write(line)
-            f.write('\r\n')
-        for side in [0, 1]:
-            name = u'デフォルトサーフェス%d' % side
-            value = self.to_zenkaku('%d' % self.default_surface[side])
-            line = '%s\t%s' % (name.encode('Shift_JIS'),
-                               value.encode('Shift_JIS'))
-            if encrypted:
-                line = ''.join(encrypt(encrypt(line)))
-            f.write(line)
-            f.write('\r\n')
-        for side in [0, 1]:
-            name = u'前回終了時サーフェス%d' % side
-            value = self.to_zenkaku('%d' % self.current_surface[side])
-            line = '%s\t%s' % (name.encode('Shift_JIS'),
-                               value.encode('Shift_JIS'))
-            if encrypted:
-                line = ''.join(encrypt(encrypt(line)))
-            f.write(line)
-            f.write('\r\n')
-        name = u'起動回数'
-        value = self.to_zenkaku('%d' % self.runcount)
-        line = '%s\t%s' % (name.encode('Shift_JIS'),
-                           value.encode('Shift_JIS'))
-        if encrypted:
-            line = ''.join(encrypt(encrypt(line)))
-        f.write(line)
-        f.write('\r\n')
-        for name in self.timer:
-            value = self.to_zenkaku(self.timer[name])
-            line = '%s\t%s' % (name.encode('Shift_JIS'),
-                               value.encode('Shift_JIS'))
-            if encrypted:
-                line = ''.join(encrypt(encrypt(line)))
-            f.write(line)
-            f.write('\r\n')
-        for name in self.reserved_talk:
-            value = self.to_zenkaku(self.reserved_talk[name])
-            line = '%s\t%s' % (''.join((u'次から', value, u'回目のトーク')).encode('Shift_JIS'),
-                               name.encode('Shift_JIS'))
-            if encrypted:
-                line = ''.join(encrypt(encrypt(line)))
-            f.write(line)
-            f.write('\r\n')
-        f.close()
 
     def finalize(self):
         self.get_event_response('OnSatoriUnload')
@@ -1054,7 +1053,7 @@ class Satori:
                 count = 0
             count += 1
             if count >= self.touch_threshold:
-                event = u'%s%sなでられ' % (str(ref3), str(ref4))
+                event = u'{0}{1}なでられ'.format(str(ref3), str(ref4))
                 count = 0
             self.mouse_move_count[key] = (count, time.time())
         elif event == 'OnMouseWheel':
@@ -1064,7 +1063,7 @@ class Satori:
                 count = 0
             count += 1
             if count >= 2:
-                event = u'%s%sころころ' % (str(ref3), str(ref4))
+                event = u'{0}{1}ころころ'.format(str(ref3), str(ref4))
                 count = 0
             self.mouse_wheel_count[key] = (count, time.time())
         elif event == 'OnSecondChange':
@@ -1304,10 +1303,10 @@ class Satori:
                 self.load_replace_file(path)
         buf = []
         for dir_ in dir_list:
-            dict_dir = os.path.join(self.satori_dir, dir_.lower())
+            dir_ = get_normalized_path(dir_, encode=0)
+            dict_dir = os.path.join(self.satori_dir, dir_)
             if not os.path.isdir(dict_dir):
-                if self.debug & 4:
-                    print 'satori.py: cannot read', dict_dir
+                logging.debug('satori.py: cannot read {0}'.format(dict_dir))
                 continue
             for path in list_dict(dict_dir):
                 filename = os.path.basename(path)
@@ -1359,7 +1358,7 @@ class Satori:
             self.current_reset_surface = [0, 0]
         script = self.get(name, default=None)
         if script is not None and script and script != '\\n':
-            ##print 'make("%s")' % script
+            ##logging.debug('make("{0}")'.format(script))
             return self.make(''.join((head, script, tail)))
         return None
 
@@ -1499,9 +1498,9 @@ class Satori:
                     start = match.start()
                     if start < len(self.newline) or \
                        script[start - len(self.newline):start] != self.newline:
-                       script = r'%s%s%s' % (
-                        script[:match.end()], self.newline_script,
-                        script[match.end():])
+                       script = r'{0}{1}{2}'.format(
+                           script[:match.end()], self.newline_script,
+                           script[match.end():])
                 else:
                     break
                 i = end
@@ -1517,9 +1516,9 @@ class Satori:
                     start = match.start()
                     if start < len(self.newline) or \
                        script[start - len(self.newline):start] != self.newline:
-                       script = r'%s%s%s' % (
-                        script[:match.end()], self.newline_script,
-                        script[match.end():])
+                       script = r'{0}{1}{2}'.format(
+                           script[:match.end()], self.newline_script,
+                           script[match.end():])
                 else:
                     break
                 i = end
@@ -1564,7 +1563,7 @@ class Satori:
         buf = []
         n = (ms + 25) * self.wait_percent / 100 / 50
         while n > 0:
-            buf.append(r'\w%d' % min(n, 9))
+            buf.append(r'\w{0:d}'.format(min(n, 9)))
             n -= 9
         return buf
 
@@ -1640,12 +1639,12 @@ class Satori:
                     side = 1
                 else:
                     side = 0
-                buf.append(r'\%d' % side)
+                buf.append(r'\{0:d}'.format(side))
                 if self.current_reset_surface[side]:
-                    buf.append(r'\s[%d]' % self.default_surface[side])
+                    buf.append(r'\s[{0:d}]'.format(self.default_surface[side]))
                     self.current_reset_surface[side] = 0
                 if newline[side] is not None:
-                    buf.append(r'%s' % newline[side])
+                    buf.append(r'{0}'.format(newline[side]))
             elif node[0] == NODE_ASSIGNMENT:
                 value = self.expand(node[2])
                 result = self.assign(self.expand(node[1]), value)
@@ -1673,7 +1672,7 @@ class Satori:
                     id_ = label
                 else:
                     id_ = self.expand(node[2])
-                buf.append(r'\q[%s,%s]\n' % (label, id_))
+                buf.append(r'\q[{0},{1}]\n'.format(label, id_))
                 talk = 1
             elif node[0] == NODE_OR_EXPR:
                 for i in range(1, len(node)):
@@ -1807,15 +1806,14 @@ class Satori:
 
     def get_reference(self, nodelist, history, side):
         key = self.expand(nodelist[1:-1], history)
-        buf = key ## FIXME
-        if buf[0] in [u'Ｒ', 'R']:
-            n = self.to_integer(buf[1:])
+        if key and key[0] in [u'Ｒ', 'R']:
+            n = self.to_integer(key[1:])
             if n is not None and 0 <= n < len(self.reference): ## FIXME
                 if self.reference[n] is None:
                     return ''
                 return unicode(self.reference[n])
-        elif buf[0] in [u'Ｓ', 'S']:
-            n = self.to_integer(buf[1:])
+        elif key and key[0] in [u'Ｓ', 'S']:
+            n = self.to_integer(key[1:])
             if n is not None:
                 if key in self.saori_value:
                     if self.saori_value[key] is None:
@@ -1823,14 +1821,14 @@ class Satori:
                     return unicode(self.saori_value[key])
                 else:
                     return ''
-        elif buf[0] in [u'Ｈ', 'H']:
-            ##print ''.join(('["', '", "'.join(history), '"]'))
-            n = self.to_integer(buf[1:])
+        elif key and key[0] in [u'Ｈ', 'H']:
+            ##logging.debug(''.join(('["', '", "'.join(history), '"]')))
+            n = self.to_integer(key[1:])
             if n is not None and 1 <= n < len(history) + 1: ## FIXME
                 return history[n-1]
-        n = self.to_integer(buf)
+        n = self.to_integer(key)
         if n is not None:
-            return r'\s[%d]' % (n + self.add_to_surface[side])
+            return r'\s[{0:d}]'.format(n + self.add_to_surface[side])
         if key in self.word:
             return self.expand(random.choice(self.word[key]), history,
                                side=side)
@@ -1885,7 +1883,7 @@ class Satori:
                     return self.to_zenkaku(1)
                 else:
                     return self.to_zenkaku(0)
-        return '（%s）' % key
+        return '（{0}）'.format(key)
 
     def calc_args(self, args, history, expand_only=0):
         buf = []
@@ -2197,8 +2195,7 @@ class Satori:
 
 class Shiori(Satori):
 
-    def __init__(self, dll_name, debug=0):
-        self.debug = debug
+    def __init__(self, dll_name):
         self.dll_name = dll_name
         self.saori = None
         self.saori_function = {}
@@ -2207,7 +2204,7 @@ class Shiori(Satori):
         self.saori = saori
 
     def load(self, satori_dir):
-        Satori.__init__(self, satori_dir, self.debug)
+        Satori.__init__(self, satori_dir)
         self.saori_library = SatoriSaoriLibrary(self.saori, self)
         Satori.load(self)
         return 1
@@ -2230,7 +2227,7 @@ class Shiori(Satori):
                         self.saori_function[list_[0]] = list_[1:]
                     else:
                         self.saori_function[list_[0]] = None
-                        ##print 'satori.py: cannot load %s' % list_[1]
+                        ##logging.error('satori.py: cannot load {0}'.format(list_[1]))
         self.parser.set_saori(self.saori_function.keys())
 
     def reload(self):
@@ -2249,20 +2246,20 @@ class Shiori(Satori):
         return result
 
     def show_description(self):
-        sys.stdout.write(
+        logging.info(
             'Shiori: SATORI compatible module for ninix\n'
             '        Copyright (C) 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
             '        Copyright (C) 2002-2011 by Shyouzou Sugitani\n'
-            '        Copyright (C) 2003, 2004 by Shun-ichi TAHARA\n')
+            '        Copyright (C) 2003, 2004 by Shun-ichi TAHARA')
 
     def request(self, req_string):
         if self.folder_change:
             self.change_folder()
             self.folder_change = 0
-        header = StringIO.StringIO(unicode(req_string, 'EUC-JP')) ## FIXME
+        header = unicode(req_string, 'EUC-JP').splitlines()
         req_header = {}
-        line = header.readline()
+        line = header.pop(0)
         if line:
             line = line.strip()
             req_list = line.split()
@@ -2284,9 +2281,7 @@ class Shiori(Satori):
         result = ''
         to = None
         if 'ID' in req_header:
-            if req_header['ID'] == 'charset':
-                result = 'EUC-JP' ## FIXME
-            elif req_header['ID'] == 'dms':
+            if req_header['ID'] == 'dms':
                 result = self.getdms()
             elif req_header['ID'] == 'OnAITalk':
                 result = self.getaistringrandom()
@@ -2325,9 +2320,10 @@ class Shiori(Satori):
             self.silent_time = 0
         result = 'SHIORI/3.0 200 OK\r\n' \
                  'Sender: Satori\r\n' \
-                 'Value: %s\r\n' % result
+                 'Charset: EUC-JP\r\n' \
+                 'Value: {0}\r\n'.format(result)
         if to is not None:
-            result = ''.join((result, 'Reference0: %s\r\n' % to))
+            result = ''.join((result, 'Reference0: {0}\r\n'.format(to)))
         result = ''.join((result, '\r\n'))
         return result.encode('EUC-JP') ## FIXME
 
@@ -2346,18 +2342,18 @@ class Shiori(Satori):
         default_args = self.saori_function[name][1:]
         n = len(default_args)
         for i in range(len(default_args)):
-              req = ''.join((req, 'Argument%s: %s\r\n' % (i, default_args[i])))
+              req = ''.join((req, 'Argument{0}: {1}\r\n'.format(i, default_args[i])))
         for i in range(len(args)):
               argument = args[i]
               if argument:
                   req = ''.join((req,
-                                 'Argument%s: %s\r\n' % (i + n, argument)))
+                                 'Argument{0}: {1}\r\n'.format(i + n, argument)))
         req = ''.join((req, '\r\n'))
         response = self.saori_library.request(
             self.saori_function[name][0],
             req.encode('Shift_JIS', 'ignore'))
-        header = StringIO.StringIO(response)
-        line = header.readline()
+        header = response.splitlines()
+        line = header.pop(0)
         if line:
             line = line.strip()
             if ' ' in line:
@@ -2368,10 +2364,11 @@ class Shiori(Satori):
                     continue
                 if ':' not in line:
                     continue
-                key, value = [x.strip() for x in line.split(':', 1)]
+                key, value = line.split(':', 1)
+                key = key.strip()
                 if key:
                     saori_header.append(key)
-                    saori_value[key] = unicode(value, 'Shift_JIS', 'ignore')
+                    saori_value[key] = unicode(value, 'Shift_JIS', 'ignore').strip()
         for key in saori_value:
             if not key.startswith('Value'):
                 continue
@@ -2387,7 +2384,7 @@ class Shiori(Satori):
             return ''
 
 
-class SatoriSaoriLibrary:
+class SatoriSaoriLibrary(object):
 
     def __init__(self, saori, satori):
         self.saori_list = {}
@@ -2416,8 +2413,8 @@ class SatoriSaoriLibrary:
         return result
 
 
-def open(top_dir, debug=0):
-    satori = Satori(top_dir, debug)
+def satori_open(top_dir):
+    satori = Satori(top_dir)
     satori.load()
     return satori
 
@@ -2440,12 +2437,12 @@ def test_ini(top_dir):
 def test_parser(path):
     parser = Parser()
     parser.read(path)
-    for name, talk in parser.talk.iteritems():
+    for name, talk in parser.talk.items():
         for node_list in talk:
             print u'＊', name
             parser.print_nodelist(node_list)
             print
-    for name, word in parser.word.iteritems():
+    for name, word in parser.word.items():
         print u'＠', name
         for node_list in word:
             print u'>>>', test_expand(node_list)
