@@ -14,7 +14,7 @@
 import random
 
 
-class EntryDatabase:
+class EntryDatabase(object):
 
     def __init__(self, db=None):
         self.__db = db or {}
@@ -46,5 +46,6 @@ def test():
         print key, entry_db.get(key)
     print 'is_empty() =', entry_db.is_empty()
 
+
 if __name__ == '__main__':
     test()
