@@ -12,6 +12,7 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
+import logging
 import os
 import re
 import sys
@@ -20,13 +21,14 @@ import ninix.config
 import ninix.alias
 import ninix.dll
 
-NINIX_HOME = '~/.ninix'
 
-def print_error(message):
-    sys.stderr.write(''.join((message, '\n')))
+PLUGIN_STANDARD = (2.0, 2.3)
 
 def get_ninix_home():
-    return os.path.expanduser(NINIX_HOME)
+    return os.path.join(os.path.expanduser('~'), '.ninix')
+
+def get_archive_dir():
+    return os.path.join(get_ninix_home(), 'archive')
 
 def get_pango_fontrc():
     return os.path.join(get_ninix_home(), 'pango_fontrc')
@@ -34,6 +36,14 @@ def get_pango_fontrc():
 def get_preferences():
     return os.path.join(get_ninix_home(), 'preferences')
 
+def get_normalized_path(path, encode=1):
+    path = path.replace('\\', '/')
+    if encode:
+        path = path.encode('utf-8')
+    if not os.path.isabs(path): # XXX
+        path = path.lower()
+    return os.path.normpath(path)
+
 def load_config():
     if not os.path.exists(get_ninix_home()):
         return None
@@ -45,16 +55,17 @@ def load_config():
     kinoko = search_kinoko()
     return ghosts, balloons, plugins, nekoninni, katochan, kinoko
 
-def get_shiori(path):
+def get_shiori():
     table = {}
-    shiori_lib = ninix.dll.Library('shiori', path, saori_lib=None)
+    shiori_lib = ninix.dll.Library('shiori', saori_lib=None)
+    path = ninix.dll.get_path()
     for filename in os.listdir(path):
-        if is_readable(os.path.join(path, filename)):
+        if os.access(os.path.join(path, filename), os.R_OK):
             name = None
-            if filename.endswith('.py'):
-                name =  filename[:-3]
-            elif filename.endswith('.pyc'):
-                name =  filename[:-4]
+            basename, ext = os.path.splitext(filename)
+            ext = ext.lower()
+            if ext in ['.py', '.pyc']:
+                name = basename
             if name and name not in table:
                 shiori = shiori_lib.request(('', name))
                 if shiori:
@@ -63,7 +74,7 @@ def get_shiori(path):
 
 def search_ghosts(target=None):
     home_dir = get_ninix_home()
-    ghosts = []
+    ghosts = {}
     if target:
         dirlist = []
         dirlist.extend(target)
@@ -72,8 +83,7 @@ def search_ghosts(target=None):
             dirlist = os.listdir(os.path.join(home_dir, 'ghost'))
         except OSError:
             dirlist = []
-    shiori_table = get_shiori(
-        os.path.join(os.environ['PYTHONPATH'], 'ninix/dll'))
+    shiori_table = get_shiori()
     for subdir in dirlist:
         prefix = os.path.join(home_dir, 'ghost', subdir)
         ghost_dir = os.path.join(prefix, 'ghost', 'master')
@@ -84,7 +94,7 @@ def search_ghosts(target=None):
         # find a pseudo AI, shells, and a built-in balloon
         candidate = {'name': '', 'score': 0}
         # SHIORI compatible modules
-        for name, shiori in shiori_table.iteritems():
+        for name, shiori in shiori_table.items():
             score = int(shiori.find(ghost_dir, shiori_dll))
             if score > candidate['score']:
                 candidate['name'] = name
@@ -93,15 +103,11 @@ def search_ghosts(target=None):
         if candidate['score'] == 0:
             continue
         shiori_name = candidate['name']
-        if desc.get('name') == 'default':
-            pos = 0
-        else:
-            pos = len(ghosts)
+        pos = 0 if desc.get('name') == 'default' else len(ghosts)
         use_makoto = find_makoto_dll(ghost_dir)
         ## FIXME: check surface_set
-        ghosts.insert(pos, (
-            desc, ghost_dir, use_makoto, surface_set, prefix,
-            shiori_dll, shiori_name))
+        ghosts[subdir] = (desc, ghost_dir, use_makoto, surface_set, prefix,
+                          shiori_dll, shiori_name)
     return ghosts
 
 def search_balloons():
@@ -111,7 +117,7 @@ def search_balloons():
         dirlist = os.listdir(balloon_dir)
     except OSError:
         dirlist = []
-    buf = []
+    buf = {}
     for subdir in dirlist:
         path = os.path.join(balloon_dir, subdir)
         if not os.path.isdir(path):
@@ -123,15 +129,12 @@ def search_balloons():
         if not balloon_info:
             continue
         if 'balloon_dir' in balloon_info: # XXX
-            print 'Oops: key confliction'
+            logging.warninig('Oops: balloon id confliction')
             continue
         else:
             balloon_info['balloon_dir'] = (subdir, ninix.config.null_config())
-        if desc.get('name') == 'default':
-            pos = 0
-        else:
-            pos = len(buf)
-        buf.insert(pos, (desc, balloon_info))
+        pos = 0 if desc.get('name') == 'default' else len(buf)
+        buf[subdir] = (desc, balloon_info)
     return buf
 
 def search_plugins():
@@ -210,50 +213,44 @@ def read_kinoko_ini(top_dir):
     kinoko['baseadjust'] = 0
     kinoko['extractpath'] = None
     kinoko['nayuki'] = None
-    if is_readable(path):
-        f = open(path)
-        line = f.readline()
-        if not line.strip() or line.strip() != '[KINOKO]':
-            f.close()
-            return None
-        lineno = 0
-        error = None
-        for line in f:
-            lineno += 1
-            if line.endswith(chr(0)): # XXX
-                line = line[:-1]
-            if not line.strip():
-                continue
-            if '=' not in line:
-                error = 'line %d: syntax error' % lineno
-                break
-            name, value = [x.strip() for x in line.split('=', 1)]
-            if name in ['title', 'ghost', 'category']:
-                kinoko[name] = unicode(value, 'Shift_JIS', 'ignore')
-            elif name in ['offsetx', 'offsety']:
-                kinoko[name] = int(value)
-            elif name in ['base', 'animation', 'extractpath']:
-                kinoko[name] = value
-            elif name in ['ontop', 'baseposition', 'baseadjust']:
-                kinoko[name] = int(value)
-        f.close()
+    if os.access(path, os.R_OK):
+        with open(path) as f:
+            line = f.readline()
+            if not line.strip() or line.strip() != '[KINOKO]':
+                return None
+            lineno = 0
+            error = None
+            for line in f:
+                lineno += 1
+                if line.endswith(chr(0)): # XXX
+                    line = line[:-1]
+                if not line.strip():
+                    continue
+                if '=' not in line:
+                    error = 'line {0:d}: syntax error'.format(lineno)
+                    break
+                name, value = [x.strip() for x in line.split('=', 1)]
+                if name in ['title', 'ghost', 'category']:
+                    kinoko[name] = unicode(value, 'Shift_JIS', 'ignore')
+                elif name in ['offsetx', 'offsety']:
+                    kinoko[name] = int(value)
+                elif name in ['base', 'animation', 'extractpath']:
+                    kinoko[name] = value
+                elif name in ['ontop', 'baseposition', 'baseadjust']:
+                    kinoko[name] = int(value)
         if error:
-            sys.stderr.write('Error: %s\n%s (skipped)\n' % (error, path))
+            logging.error('Error: {0}\n{1} (skipped)'.format(error, path))
             return None
-    if kinoko['title']:
-        return kinoko
-    else:
-        return None
+    return kinoko if kinoko['title'] else None
 
 def read_profile_txt(top_dir):
     path = os.path.join(top_dir, 'profile.txt')
     name = None
-    if is_readable(path):
-        f = open(path)
-        line = f.readline()
+    if os.access(path, os.R_OK):
+        with open(path) as f:
+            line = f.readline()
         if line:
             name = unicode(line.strip(), 'Shift_JIS', 'ignore')
-        f.close()
     if name:
         return (name, top_dir) ## FIXME
     else:
@@ -263,96 +260,88 @@ def read_katochan_txt(top_dir):
     path = os.path.join(top_dir, 'katochan.txt')
     katochan = {}
     katochan['dir'] = top_dir
-    if is_readable(path):
-        f = open(path)
-        name = None
-        lineno = 0
-        error = None
-        for line in f:
-            lineno += 1
-            if not line.strip():
-                continue
-            if line.startswith('#'):
-                name = line[1:].strip()
-                continue
-            elif not name:
-                error = 'line %d: syntax error' % lineno
-                break
-            else:
-                value = line.strip()
-                if name in ['name', 'category']:
-                    katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
-                if name.startswith('before.script') or \
-                        name.startswith('hit.script') or \
-                        name.startswith('after.script') or \
-                        name.startswith('end.script') or \
-                        name.startswith('dodge.script'):
-                    ## FIXME: should be array
-                    katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
-                elif name in ['before.fall.speed', 'before.slide.magnitude',
-                              'before.slide.sinwave.degspeed',
-                              'before.appear.ofset.x',
-                              'before.appear.ofset.y',
-                              'hit.waittime', 'hit.ofset.x', 'hit.ofset.y',
-                              'after.fall.speed', 'after.slide.magnitude',
-                              'after.slide.sinwave.degspeed']:
-                    katochan[name] = int(value)
-                elif name in ['target',
-                              'before.fall.type', 'before.slide.type',
-                              'before.wave', 'before.wave.loop',
-                              'before.appear.direction',
-                              'hit.wave', 'hit.wave.loop',
-                              'after.fall.type', 'after.slide.type',
-                              'after.wave', 'after.wave.loop',
-                              'end.wave', 'end.wave.loop',
-                              'end.leave.direction',
-                              'dodge.wave', 'dodge.wave.loop']:
-                    katochan[name] = value
+    if os.access(path, os.R_OK):
+        with open(path) as f:
+            name = None
+            lineno = 0
+            error = None
+            for line in f:
+                lineno += 1
+                if not line.strip():
+                    continue
+                if line.startswith('#'):
+                    name = line[1:].strip()
+                    continue
+                elif not name:
+                    error = 'line {0:d}: syntax error'.format(lineno)
+                    break
                 else:
-                    name = None
-        f.close()
+                    value = line.strip()
+                    if name in ['name', 'category']:
+                        katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
+                    if name.startswith('before.script') or \
+                            name.startswith('hit.script') or \
+                            name.startswith('after.script') or \
+                            name.startswith('end.script') or \
+                            name.startswith('dodge.script'):
+                        ## FIXME: should be array
+                        katochan[name] = unicode(value, 'Shift_JIS', 'ignore')
+                    elif name in ['before.fall.speed',
+                                  'before.slide.magnitude',
+                                  'before.slide.sinwave.degspeed',
+                                  'before.appear.ofset.x',
+                                  'before.appear.ofset.y',
+                                  'hit.waittime', 'hit.ofset.x', 'hit.ofset.y',
+                                  'after.fall.speed', 'after.slide.magnitude',
+                                  'after.slide.sinwave.degspeed']:
+                        katochan[name] = int(value)
+                    elif name in ['target',
+                                  'before.fall.type', 'before.slide.type',
+                                  'before.wave', 'before.wave.loop',
+                                  'before.appear.direction',
+                                  'hit.wave', 'hit.wave.loop',
+                                  'after.fall.type', 'after.slide.type',
+                                  'after.wave', 'after.wave.loop',
+                                  'end.wave', 'end.wave.loop',
+                                  'end.leave.direction',
+                                  'dodge.wave', 'dodge.wave.loop']:
+                        katochan[name] = value
+                    else:
+                        name = None
         if error:
-            sys.stderr.write('Error: %s\n%s (skipped)\n' % (error, path))
+            logging.error('Error: {0}\n{1} (skipped)'.format(error, path))
             return None
-    if katochan['name']:
-        return katochan
-    else:
-        return None
+    return katochan if katochan['name'] else None
 
 def read_descript_txt(top_dir):
     path = os.path.join(top_dir, 'descript.txt')
-    if is_readable(path):
+    if os.access(path, os.R_OK):
         return ninix.config.create_from_file(path)
     return None
 
 def read_install_txt(top_dir):
     path = os.path.join(top_dir, 'install.txt')
-    if is_readable(path):
+    if os.access(path, os.R_OK):
         return ninix.config.create_from_file(path)
     return None
 
 def read_alias_txt(top_dir):
     path = os.path.join(top_dir, 'alias.txt')
-    if is_readable(path):
+    if os.access(path, os.R_OK):
         return ninix.alias.create_from_file(path)
     return None
 
 def find_makoto_dll(top_dir):
-    if is_readable(os.path.join(top_dir, 'makoto.dll')):
-        return 1
-    return 0
+    return 1 if os.access(os.path.join(top_dir, 'makoto.dll'), os.R_OK) else 0
 
 def find_surface_set(top_dir):
     desc = read_descript_txt(os.path.join(top_dir, 'ghost', 'master'))
-    if desc:
-        shell_name = desc.get('name')
-    else:
-        shell_name = None
+    shell_name = desc.get('name') if desc else None
     if not shell_name:
         inst = read_install_txt(top_dir)
         if inst:
             shell_name = inst.get('name')
-    surface_set = []
+    surface_set = {}
     shell_dir = os.path.join(top_dir, 'shell')
     for name, desc, subdir in find_surface_dir(shell_dir):
         surface_dir = os.path.join(shell_dir, subdir)
@@ -361,8 +350,8 @@ def find_surface_set(top_dir):
            'surface0' in surface_info and 'surface10' in surface_info:
             if alias is None:
                 alias = read_alias_txt(surface_dir)
-            surface_set.append((name, surface_dir, desc, alias, surface_info,
-                                tooltips))
+            surface_set[subdir] = (name, surface_dir, desc, alias,
+                                   surface_info, tooltips)
     return shell_name, surface_set
 
 def find_surface_dir(top_dir):
@@ -371,10 +360,11 @@ def find_surface_dir(top_dir):
     if os.path.exists(path):
         config = ninix.config.create_from_file(path)
         for name, subdir in config.items():
-            desc = read_descript_txt(os.path.join(top_dir, subdir.lower()))
+            subdir = subdir.lower()
+            desc = read_descript_txt(os.path.join(top_dir, subdir))
             if desc is None:
                 desc = ninix.config.null_config()
-            buf.append((name, desc, subdir.lower()))
+            buf.append((name, desc, subdir))
     else:
         try:
             dirlist = os.listdir(top_dir)
@@ -400,30 +390,30 @@ def read_surface_info(surface_dir):
     path = os.path.join(surface_dir, 'alias.txt')
     if os.path.exists(path):
         dic = ninix.alias.create_from_file(path)
-        for basename, alias in dic.iteritems():
+        for basename, alias in dic.items():
             if basename.startswith('surface'):
                 filename_alias[alias] = basename
     # find png image and associated configuration file
     for filename in filelist:
-        basename, suffix = os.path.splitext(filename)
+        basename, ext = os.path.splitext(filename)
         if basename in filename_alias:
             match = re_surface.match(
-                ''.join((filename_alias[basename], suffix)))
+                ''.join((filename_alias[basename], ext)))
         else:
             match = re_surface.match(filename)
         if not match:
             continue
         img = os.path.join(surface_dir, filename)
-        if not is_readable(img):
+        if not os.access(img, os.R_OK):
             continue
         key = ''.join(('surface', str(int(match.group(1)))))
         txt = os.path.join(surface_dir, ''.join((basename, 's.txt')))
-        if is_readable(txt):
+        if os.access(txt, os.R_OK):
             config = ninix.config.create_from_file(txt)
         else:
             config = ninix.config.null_config()
         txt = os.path.join(surface_dir, ''.join((basename, 'a.txt')))
-        if is_readable(txt):
+        if os.access(txt, os.R_OK):
             config.update(ninix.config.create_from_file(txt))
         surface[key] = (img, config)
     # find surfaces.txt
@@ -436,7 +426,9 @@ def read_surface_info(surface_dir):
             tooltips = config
         elif key.startswith('surface'):
             try:
-                img, null_config = surface[key]
+                img, prev_config = surface[key]
+                prev_config.update(config)
+                config = prev_config
             except KeyError:
                 img = None
             surface[key] = (img, config)
@@ -444,7 +436,7 @@ def read_surface_info(surface_dir):
     for key, (img, config) in surface.items():
         for key, method, filename, x, y in list_surface_elements(config):
             filename = filename.lower()
-            basename, suffix = os.path.splitext(filename)
+            basename, ext = os.path.splitext(filename)
             if basename not in surface:
                 surface[basename] = (os.path.join(surface_dir, filename),
                                      ninix.config.null_config())
@@ -454,74 +446,79 @@ def read_surfaces_txt(surface_dir):
     config_list = []
     path = os.path.join(surface_dir, 'surfaces.txt')
     try:
-        f = open(path)
+        with open(path) as f:
+            alias_buffer = []
+            tooltips = {}
+            charset = 'Shift_JIS'
+            buf = []
+            key = None
+            opened = False
+            for line in f:
+                if line.startswith('#') or line.startswith('//'):
+                    continue
+                # '\x81\x40': full-width space in Shift_JIS
+                temp = line.replace('\x81\x40', '').strip()
+                if not temp:
+                    continue
+                if temp.startswith('charset'):
+                    try:
+                        charset = line.split(',', 1)[1].strip()
+                    except:
+                        pass
+                    continue
+                if key is None:
+                    if temp.endswith('{'):
+                        key = temp[:-1]
+                        opened = True
+                    else:
+                        key = temp
+                elif temp == '{':
+                    opened = True
+                elif temp.endswith('}'):
+                    if temp[:-1]:
+                        buf.append(temp[:-1])
+                    if not opened:
+                        logging.error(
+                            'syntax error: unbalnced "}" in surfaces.txt.')
+                    if key in ['sakura.surface.alias', 'kero.surface.alias']:
+                        alias_buffer.append(key)
+                        alias_buffer.append('{')
+                        alias_buffer.extend(buf)
+                        alias_buffer.append('}')
+                    elif key.endswith('.tooltips'):
+                        try:
+                            key = key[:-9]
+                        except:
+                            pass
+                        value = {}
+                        for line in buf:
+                            line = line.split(',', 1)
+                            region, text = [unicode(s.strip(), charset, 'ignore') for s in line]
+                            value[region] = text
+                            tooltips[key] = value
+                    elif key.startswith('surface'):
+                        keys = key.split(',')
+                        for key in keys:
+                            if not key:
+                                continue
+                            if key.startswith('surface'):
+                                try:
+                                    key = ''.join((key[:7], str(int(key[7:]))))
+                                except ValueError:
+                                    pass
+                            else:
+                                try:
+                                    key = ''.join(('surface', str(int(key))))
+                                except ValueError:
+                                    pass
+                            config_list.append((key, ninix.config.create_from_buffer(buf)))
+                    buf = []
+                    key = None
+                    opened = False
+                else:
+                    buf.append(temp)
     except IOError:
         return config_list
-    alias_buffer = []
-    tooltips = {}
-    charset = 'Shift_JIS'
-    while 1:
-        line = f.readline()
-        if not line:
-            break
-        if line.startswith('#') or line.startswith('//'):
-            continue
-        key = line.strip()
-        if key.startswith('charset'):
-            try:
-                charset = line.split(',', 1)[1].strip()
-            except:
-                pass
-            continue
-        if not key:
-            continue
-        try:
-            while 1:
-                line = f.readline()
-                if not line:
-                    raise ValueError, 'unexpected end of file'
-                line = line.replace('\x81\x40', '').strip()
-                if not line:
-                    continue
-                elif line == '{':
-                    break
-                key = line # ignore the preceding key
-            buf = []
-            while 1:
-                line = f.readline()
-                if not line:
-                    raise ValueError, 'unexpected end of file'
-                line = line.replace('\x81\x40', '').strip()
-                if not line:
-                    continue
-                elif line == '}':
-                    break
-                buf.append(line)
-            if key in ['sakura.surface.alias', 'kero.surface.alias']:
-                alias_buffer.append(key)
-                alias_buffer.append('{')
-                alias_buffer.extend(buf)
-                alias_buffer.append('}')
-            elif key.endswith('.tooltips'):
-                try:
-                    key = key[:-9]
-                except:
-                    pass
-                value = {}
-                for line in buf:
-                    line = line.split(',', 1)
-                    region, text = [unicode(s.strip(), charset, 'ignore') for s in line]
-                    value[region] = text
-                tooltips[key] = value
-            elif key.startswith('surface'):
-                try:
-                    key = ''.join((key[:7], str(int(key[7:]))))
-                except ValueError:
-                    pass
-                config_list.append((key, ninix.config.create_from_buffer(buf)))
-        except ValueError, error:
-            print_error('%s: %s (parsing not completed)' % (path, error))
-            break
     if alias_buffer:
         config_list.append(('__alias__', ninix.alias.create_from_buffer(alias_buffer)))
     config_list.append(('__tooltips__', tooltips))
@@ -539,7 +536,8 @@ def list_surface_elements(config):
             x = int(x)
             y = int(y)
         except ValueError:
-            print_error('invalid element spec for %s: %s' % (key, config[key]))
+            logging.error(
+                'invalid element spec for {0}: {1}'.format(key, config[key]))
             continue
         buf.append((key, method, filename, x, y))
     return buf
@@ -559,13 +557,13 @@ def read_balloon_info(balloon_dir):
             continue
         img = os.path.join(balloon_dir, filename)
         if match.group(2) != 'png' and \
-           is_readable(''.join((img[-3:], 'png'))):
+           os.access(''.join((img[-3:], 'png')), os.R_OK):
                 continue
-        if not is_readable(img):
+        if not os.access(img, os.R_OK):
             continue
         key = match.group(1)
-        txt = os.path.join(balloon_dir, 'balloon%ss.txt' % key)
-        if is_readable(txt):
+        txt = os.path.join(balloon_dir, 'balloon{0}s.txt'.format(key))
+        if os.access(txt, os.R_OK):
             config = ninix.config.create_from_file(txt)
         else:
             config = ninix.config.null_config()
@@ -575,90 +573,89 @@ def read_balloon_info(balloon_dir):
         if not match:
             continue
         img = os.path.join(balloon_dir, filename)
-        if not is_readable(img):
+        if not os.access(img, os.R_OK):
             continue
         key = match.group(1)
         config = ninix.config.null_config()
         balloon[key] = (img, config)
     return balloon
 
-def read_plugin_txt(plugin_dir):
-    path = os.path.join(plugin_dir, 'plugin.txt')
+def read_plugin_txt(src_dir):
+    path = os.path.join(src_dir, 'plugin.txt')
     try:
-        f = open(path)
+        with open(path) as f:
+            charset = 'UTF-8' # default
+            standard = 0.0
+            plugin_name = startup = None
+            menu_items = []
+            error = None
+            lineno = 0
+            for line in f:
+                lineno += 1
+                if not line.strip() or line.startswith('#'):
+                    continue
+                if ':' not in line:
+                    error = 'line {0:d}: syntax error'.format(lineno)
+                    break
+                name, value = [x.strip() for x in line.split(':', 1)]
+                if name == 'charset':
+                    charset = value
+                elif name == 'standard':
+                    standard = float(value)
+                elif name == 'name':
+                    plugin_name = unicode(value, charset, 'ignore')
+                elif name == 'startup':
+                    startup_list = value.split(',')
+#                    startup_list[0] = os.path.join(plugin_dir, startup_list[0])
+                    if not os.path.exists(os.path.join(src_dir, startup_list[0])):
+                        error = 'line {0:d}: invalid program name'.format(lineno)
+                        break
+                    startup = startup_list
+                elif name == 'menuitem':
+                    menuitem_list = unicode(value, charset, 'ignore').split(',')
+                    if len(menuitem_list) < 2:
+                        error = 'line {0:d}: syntax error'.format(lineno)
+                        break
+#                    menuitem_list[1] = os.path.join(plugin_dir, menuitem_list[1])
+                    if not os.path.exists(os.path.join(src_dir, menuitem_list[1])):
+                        error = 'line {0:d}: invalid program name'.format(lineno)
+                        break
+                    menu_items.append((menuitem_list[0], menuitem_list[1:]))
+                elif name == 'directory':
+                    plugin_dir = unicode(value, charset, 'ignore').encode('utf-8') # XXX
+                else:
+                    error = 'line {0:d}: syntax error'.format(lineno)
+                    break
+            else:
+                if plugin_name is None:
+                    error = "the 'name' header field is required"
+                elif not startup and not menu_items:
+                    error = "either 'startup' or 'menuitem' header field is required"
+                elif standard < PLUGIN_STANDARD[0] or \
+                        standard > PLUGIN_STANDARD[1]:
+                    error = "standard version mismatch"
     except IOError:
         return None
-    charset = 'UTF-8' # default
-    plugin_name = startup = None
-    menu_items = []
-    error = None
-    lineno = 0
-    for line in f:
-        lineno += 1
-        if not line.strip() or line.startswith('#'):
-            continue
-        if ':' not in line:
-            error = 'line %d: syntax error' % lineno
-            break
-        name, value = [x.strip() for x in line.split(':', 1)]
-        if name == 'charset':
-            charset = value
-        elif name == 'name':
-            plugin_name = unicode(value, charset, 'ignore')
-        elif name == 'startup':
-            startup_list = value.split(',')
-            startup_list[0] = os.path.join(plugin_dir, startup_list[0])
-            if not os.path.exists(startup_list[0]):
-                error = 'line %d: invalid program name' % lineno
-                break
-            startup = startup_list
-        elif name == 'menuitem':
-            menuitem_list = unicode(value, charset, 'ignore').split(',')
-            if len(menuitem_list) < 2:
-                error = 'line %d: syntax error' % lineno
-                break
-            menuitem_list[1] = os.path.join(plugin_dir, menuitem_list[1])
-            if not os.path.exists(menuitem_list[1]):
-                error = 'line %d: invalid program name' % lineno
-                break
-            menu_items.append((menuitem_list[0], menuitem_list[1:]))
-        else:
-            error = 'line %d: syntax error' % lineno
-            break
-    else:
-        if plugin_name is None:
-            error = "the 'name' header field is required"
-        elif not startup and not menu_items:
-            error = "either 'startup' or 'menuitem' header field is required"
-    f.close()
     if error:
-        sys.stderr.write('Error: %s\n%s (skipped)\n' % (error, path))
+        sys.stderr.write('Error: {0}\n{1} (skipped)\n'.format(error, path))
         return None
     return plugin_name, plugin_dir, startup, menu_items
 
-def is_readable(path):
-    try:
-        open(path).read(64)
-    except IOError:
-        return 0
-    return 1
-    
+
 ###   TEST   ###
 
 def test():
+    head, tail = os.path.split(sys.path[0])
+    sys.path[0] = head # XXX
     import locale
     locale.setlocale(locale.LC_ALL, '')
-    global NINIX_HOME
-    try:
-        NINIX_HOME = os.environ['NINIX_HOME']
-    except KeyError:
-        pass
     config = load_config()
     if config is None:
         raise SystemExit, 'Home directory not found.\n'
     ghosts, balloons, plugins, nekoninni, katochan, kinoko = config
     # ghosts
-    for desc, shiori_dir, use_makoto, surface_set, prefix, shiori_dll, shiori_name in ghosts:
+    for key in ghosts:
+        desc, shiori_dir, use_makoto, surface_set, prefix, shiori_dll, shiori_name = ghosts[key]
         print 'GHOST', '=' * 50
         print prefix
         print str(desc).encode('utf-8', 'ignore')
@@ -667,19 +664,19 @@ def test():
         print shiori_name
         print 'use_makoto =', use_makoto
         if surface_set:
-            for name, surface_dir, desc, alias, surface in surface_set:
+            for name, surface_dir, desc, alias, surface, tooltips in surface_set.values():
                 print '-' * 50
                 print 'surface:', name.encode('utf-8', 'ignore')
                 print str(desc).encode('utf-8', 'ignore')
-                for k, v in surface.iteritems():
+                for k, v in surface.items():
                     print k, '=', v[0]
                     print str(v[1]).encode('utf-8', 'ignore')
                 if alias:
                     buf = []
-                    for k, v in alias.iteritems():
+                    for k, v in alias.items():
                         if k in ['sakura.surface.alias', 'kero.surface.alias']:
                             print ''.join((k, ':'))
-                            for alias_id, alias_list in v.iteritems():
+                            for alias_id, alias_list in v.items():
                                 print alias_id, \
                                       ''.join(('= [', ', '.join(alias_list), ']'))
                             print
@@ -691,10 +688,11 @@ def test():
                             print k, '=', v
                         print
     # balloons
-    for desc, balloon in balloons:
+    for key in balloons:
+        desc, balloon = balloons[key]
         print 'BALLOON', '=' * 50
         print str(desc).encode('utf-8', 'ignore')
-        for k, v in balloon.iteritems():
+        for k, v in balloon.items():
             print k, '=', v[0]
             print str(v[1]).encode('utf-8', 'ignore')
     # plugins
@@ -704,12 +702,13 @@ def test():
         if startup:
             print 'startup =', ''.join(('["', '", "'.join(startup), '"]'))
         for label, argv in menu_items:
-            print "menuitem '%s' =" % label.encode('utf-8', 'ignore'),
+            print "menuitem '{0}' =".format(label.encode('utf-8', 'ignore')),
             print ''.join(('["', '", "'.join(argv), '"]'))
     ## FIXME
     # kinoko
     # nekoninni
     # katochan
 
+
 if __name__ == '__main__':
     test()
