@@ -14,15 +14,14 @@
 import os
 import re
 
-if 'DISPLAY' in os.environ:
-    import gtk
-    import ninix.pix
+import gtk
+import ninix.pix
 
 
-class Menu:
+class Menu(object):
 
-    def __init__(self, callback):
-        self.callback = callback
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         ui_info = '''
         <ui>
           <popup name='popup'>
@@ -39,6 +38,7 @@ class Menu:
             <menuitem action='Update'/>
             <menuitem action='Vanish'/>
             <menuitem action='Preferences'/>
+            <menuitem action='Console'/>
             <menuitem action='Manager'/>
             </menu>
             <separator/>
@@ -80,37 +80,41 @@ class Menu:
                 None, 1, 1],
             'Options/Update': [
                 ('Update', None, _('Network Update(_U)'), None,
-                 '', lambda *a: self.callback['network_update']()),
+                 '', lambda *a: self.request_parent('NOTIFY', 'network_update')),
                 None, 1, 1],
             'Options/Vanish': [
                 ('Vanish', None, _('Vanish(_F)'), None,
-                 '', lambda *a: self.callback['vanish_sakura']()),
+                 '', lambda *a: self.request_parent('NOTIFY', 'vanish_sakura')),
                 None, 1, 1],
             'Options/Preferences': [
                 ('Preferences', None, _('Preferences...(_O)'), None,
-                 '', lambda *a: self.callback['edit_preferences']()),
+                 '', lambda *a: self.request_parent('NOTIFY', 'edit_preferences')),
+                None, 1, 1],
+            'Options/Console': [
+                ('Console', None, _('Console(_C)'), None,
+                 '', lambda *a: self.request_parent('NOTIFY', 'open_console')),
                 None, 1, 1],
             'Options/Manager': [
                 ('Manager', None, _('Ghost Manager(_M)'), None,
-                 '', lambda *a: self.callback['open_ghost_manager']()),
+                 '', lambda *a: self.request_parent('NOTIFY', 'open_ghost_manager')),
                 None, 1, 1],
             'Information': [
                 ('Information', None, _('Information(_I)'), None),
                 None, 1, 1],
             'Information/Usage': [
                 ('Usage', None, _('Usage graph(_A)'), None,
-                 '', lambda *a: self.callback['show_usage']()),
+                 '', lambda *a: self.request_parent('NOTIFY', 'show_usage')),
                 None, 1, 1],
             'Information/Version': [
                 ('Version', None, _('Version(_V)'), None,
-                 '', lambda *a: self.callback['about']()),
+                 '', lambda *a: self.request_parent('NOTIFY', 'about')),
                 None, 1, 1],
             'Close': [
                 ('Close', None, _('Close(_W)'), None,
-                 '', lambda *a: self.callback['close_sakura']()),
+                 '', lambda *a: self.request_parent('NOTIFY', 'close_sakura')),
                 None, 1, 1],
             'Quit': [('Quit', None, _('Quit(_Q)'), None,
-                      '', lambda *a: self.callback['close_all']()),
+                      '', lambda *a: self.request_parent('NOTIFY', 'close_all')),
                      None, 1, 1],
             'Change': [
                 ('Change', None, _('Change(_G)'), None),
@@ -129,7 +133,8 @@ class Menu:
                 None, 1, 1],
             'Stick': [
                 ('Stick', None, _('Stick(_Y)'), None,
-                 '', self.callback['stick_window'], False),
+                 '', lambda *a: self.request_parent('NOTIFY', 'stick_window'),
+                 False),
                 None, 1, 1],
             'Nekodorif': [
                 ('Nekodorif', None, _('Nekodorif(_N)'), None),
@@ -148,10 +153,8 @@ class Menu:
         self.__fontcolor = {'normal': '#000000', 'prelight': '#ffffff'} ## FIXME
         self.__sidebar_width = 0
         actions = gtk.ActionGroup('Actions')
-        entry = []
-        for key, value in self.__menu_list.iteritems():
-            if key != 'Stick':
-                entry.append(value[0])
+        entry = [value[0] for key, value in self.__menu_list.items() \
+                     if key != 'Stick']
         actions.add_actions(tuple(entry))
         actions.add_toggle_actions(tuple([self.__menu_list['Stick'][0]]))
         self.ui_manager = gtk.UIManager()
@@ -159,9 +162,12 @@ class Menu:
         self.ui_manager.add_ui_from_string(ui_info)
         self.__popup_menu = self.ui_manager.get_widget('/popup')
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def set_fontcolor(self, background, foreground):
-        self.__fontcolor['normal'] = '#%02x%02x%02x' % background
-        self.__fontcolor['prelight'] = '#%02x%02x%02x' % foreground
+        self.__fontcolor['normal'] = '#{0:02x}{1:02x}{2:02x}'.format(*background)
+        self.__fontcolor['prelight'] = '#{0:02x}{1:02x}{2:02x}'.format(*foreground)
 
     def set_pixmap(self, path_background, path_sidebar, path_foreground):
         pixbuf = None
@@ -277,8 +283,11 @@ class Menu:
                         item = gtk.CheckMenuItem(name)
                         item.set_name('popup menu item')
                         item.set_active(bool(state))
-                        item.connect('activate', self.callback['toggle_bind'],
-                                     (index, key))
+                        item.connect(
+                            'activate',
+                            lambda a, k: self.request_parent(
+                                'NOTIFY', 'toggle_bind', k),
+                            (index, key))
                     else:
                         item = gtk.SeparatorMenuItem()
                     item.show()
@@ -321,12 +330,12 @@ class Menu:
                 elif key == 'Portal':
                     name_list = [] # same as 'kero'
                 elif key == 'Recommend':
-                    name_list = ['char%d.recommendbuttoncaption' % side]
+                    name_list = ['char{0:d}.recommendbuttoncaption'.format(side)]
             else:
                 name_list = self.__ui[key][0][side]
             if name_list: # caption
                 for name in name_list:
-                    caption = self.callback['getstring'](name)
+                    caption = self.request_parent('GET', 'getstring', name)
                     if caption:
                         break
                 if caption:
@@ -340,7 +349,7 @@ class Menu:
                 name_list = self.__ui[key][2][side]
             if name_list: # visible
                 for name in name_list:
-                    visible = self.callback['getstring'](name)
+                    visible = self.request_parent('GET', 'getstring', name)
                     if visible is not None:
                         break
                 if visible == '0':
@@ -384,26 +393,27 @@ class Menu:
 
     def popup(self, button, side):
         if side > 1:
-            string = 'char%d' % side
+            string = 'char{0:d}'.format(side)
         else:
             assert side in [0, 1] ## FIXME
             string = ['sakura', 'kero'][side]
         string = ''.join((string, '.popupmenu.visible'))
-        if self.callback['getstring'](string) == '0':
+        if self.request_parent('GET', 'getstring', string) == '0':
             return
         self.__update_ui(side)
         if side == 0:
-            portal = self.callback['getstring']('sakura.portalsites')
+            portal = self.request_parent(
+                'GET', 'getstring', 'sakura.portalsites')
         else:
             portal = None
         self.__set_portal_menu(side, portal)
         if side > 1:
-            string = 'char%d' % side
+            string = 'char{0:d}'.format(side)
         else:
             assert side in [0, 1] ## FIXME
             string = ['sakura', 'kero'][side]
         string = ''.join((string, '.recommendsites'))
-        recommend = self.callback['getstring'](string)
+        recommend = self.request_parent('GET', 'getstring', string)
         self.__set_recommend_menu(recommend)
         self.__set_ghost_menu()
         self.__set_shell_menu()
@@ -498,9 +508,11 @@ class Menu:
                         #else:
                         #    bannar = None
                         item = gtk.MenuItem(title)
-                        item.connect('activate',
-                                     self.callback['notify_site_selection'],
-                                     (title, url))
+                        item.connect(
+                            'activate',
+                            lambda a, i: self.request_parent(
+                                'NOTIFY', 'notify_site_selection', i),
+                            (title, url))
                     menu.add(item)
                     item.show()
                 menuitem = self.ui_manager.get_widget(
@@ -533,9 +545,11 @@ class Menu:
                     #else:
                     #    bannar = None
                     item = gtk.MenuItem(title)
-                    item.connect('activate',
-                                 self.callback['notify_site_selection'],
-                                 (title, url))
+                    item.connect(
+                        'activate',
+                        lambda a, i: self.request_parent(
+                            'NOTIFY', 'notify_site_selection', i),
+                        (title, url))
                 menu.add(item)
                 item.show()
             menuitem =  self.ui_manager.get_widget(
@@ -549,9 +563,9 @@ class Menu:
 
     def __update_ghost_menu(self, set_list, summon):
         if summon:
-            callback = self.callback['start_sakura']
+            menu_type = 'start_sakura'
         else:
-            callback = self.callback['select_sakura']
+            menu_type = 'select_sakura'
         ghost_menu = gtk.Menu()
         for i in range(len(set_list)):
             if set_list[i] is None:
@@ -574,7 +588,10 @@ class Menu:
                 item.set_sensitive(False)
             if len(shell_list) <= 1:
                 shell_name, value = shell_list[0]
-                item.connect('activate', callback, value)
+                item.connect(
+                    'activate',
+                    lambda a, v: self.request_parent('NOTIFY', menu_type, v),
+                    (value))
             else:
                 submenu = gtk.Menu()
                 submenu.set_name('popup menu')
@@ -582,7 +599,11 @@ class Menu:
                 for shell_name, value in shell_list:
                     item = gtk.MenuItem(shell_name)
                     item.set_name('popup menu item')
-                    item.connect('activate', callback, value)
+                    item.connect(
+                        'activate',
+                        lambda a, v: self.request_parent(
+                            'NOTIFY', menu_type, v),
+                        (value))
                     item.show()
                     submenu.append(item)
         path = 'Summon' if summon else 'Change'
@@ -591,18 +612,21 @@ class Menu:
         self.__menu_list[path][3] = 1
 
     def __set_ghost_menu(self):
-        ghosts = self.callback['get_ghost_list']()
+        ghosts = self.request_parent('GET', 'get_ghost_list')
         self.__update_ghost_menu(ghosts, False)
         self.__update_ghost_menu(ghosts, True)
 
     def __set_shell_menu(self):
-        shell_list = self.callback['get_shell_list']()
+        shell_list = self.request_parent('GET', 'get_shell_list')
         shell_menu = gtk.Menu()
         for i in range(len(shell_list)):
             shell_name, value, working = shell_list[i]
             item = gtk.MenuItem(shell_name)
             item.set_name('popup menu item')
-            item.connect('activate', self.callback['select_shell'], value)
+            item.connect(
+                'activate',
+                lambda a, v: self.request_parent('NOTIFY', 'select_shell', v),
+                (value))
             item.show()
             if working:
                 item.set_sensitive(False)
@@ -612,17 +636,20 @@ class Menu:
         self.__menu_list['Shell'][3] = 1
 
     def __set_balloon_menu(self):
-        balloon_list = self.callback['get_balloon_list']()
+        balloon_list = self.request_parent('GET', 'get_balloon_list')
         balloon_menu = gtk.Menu()
         group = None
         for name, directory in balloon_list:
             item = group = gtk.RadioMenuItem(group, name)
             item.set_name('popup menu item')
-            hid = item.connect('activate',
-                               self.callback['select_balloon'], directory)
+            hid = item.connect(
+                'activate',
+                lambda w, d: self.request_parent(
+                    'NOTIFY', 'select_balloon', w, d),
+                directory)
             item.show()
             balloon_menu.append(item)
-            if directory == self.callback['get_current_balloon_directory']():
+            if directory == self.request_parent('GET', 'get_current_balloon_directory'):
                 item.handler_block(hid)
                 item.activate()
                 item.handler_unblock(hid)
@@ -631,7 +658,7 @@ class Menu:
         self.__menu_list['Balloon'][3] = 1
 
     def __set_plugin_menu(self):
-        plugin_list = self.callback['get_plugin_list']()
+        plugin_list = self.request_parent('GET', 'get_plugin_list')
         plugin_menu = gtk.Menu()
         for i in range(len(plugin_list)):
             name = plugin_list[i]['name']
@@ -642,7 +669,11 @@ class Menu:
             item_list = plugin_list[i]['items']
             if len(item_list) <= 1:
                 label, value = item_list[0]
-                item.connect('activate', self.callback['select_plugin'], value)
+                item.connect(
+                    'activate',
+                    lambda a, v: self.request_parent(
+                        'NOTIFY', 'select_plugin', v),
+                    (value))
                 ##if working:
                 ##    item.set_sensitive(False)
             else:
@@ -652,8 +683,11 @@ class Menu:
                 for label, value in item_list:
                     item = gtk.MenuItem(label)
                     item.set_name('popup menu item')
-                    item.connect('activate', self.callback['select_plugin'],
-                                 value)
+                    item.connect(
+                        'activate',
+                        lambda a, v: self.request_parent(
+                            'NOTIFY',  'select_plugin', v),
+                        (value))
                     item.show()
                     ##if working:
                     ##    item.set_sensitive(False)
@@ -663,7 +697,7 @@ class Menu:
         self.__menu_list['Plugin'][3] = 1
 
     def __set_nekodorif_menu(self):
-        nekodorif_list = self.callback['get_nekodorif_list']()
+        nekodorif_list = self.request_parent('GET', 'get_nekodorif_list')
         nekodorif_menu = gtk.Menu()
         for i in range(len(nekodorif_list)):
             name = nekodorif_list[i]['name']
@@ -671,8 +705,11 @@ class Menu:
             item.set_name('popup menu item')
             item.show()
             nekodorif_menu.append(item)
-            item.connect('activate', self.callback['select_nekodorif'],
-                         nekodorif_list[i]['dir'])
+            item.connect(
+                'activate',
+                lambda a, n: self.request_parent(
+                    'NOTIFY', 'select_nekodorif', n),
+                (nekodorif_list[i]['dir']))
             ##if working:
             ##    item.set_sensitive(False)
         menuitem = self.ui_manager.get_widget(
@@ -681,7 +718,7 @@ class Menu:
         self.__menu_list['Nekodorif'][3] = 1
 
     def __set_kinoko_menu(self):
-        kinoko_list = self.callback['get_kinoko_list']()
+        kinoko_list = self.request_parent('GET', 'get_kinoko_list')
         kinoko_menu = gtk.Menu()
         for i in range(len(kinoko_list)):
             name = kinoko_list[i]['title']
@@ -689,8 +726,11 @@ class Menu:
             item.set_name('popup menu item')
             item.show()
             kinoko_menu.append(item)
-            item.connect('activate', self.callback['select_kinoko'],
-                         kinoko_list[i])
+            item.connect(
+                'activate',
+                lambda a, k: self.request_parent(
+                    'NOTIFY', 'select_kinoko', k),
+                (kinoko_list[i]))
             ##if working:
             ##    item.set_sensitive(False)
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', 'Kinoko')))
@@ -705,5 +745,6 @@ class Menu:
 def test():
     pass
 
+
 if __name__ == '__main__':
     test()
