@@ -48,19 +48,20 @@
 
 import os
 import random
+import logging
 
-if 'DISPLAY' in os.environ:
-    import gtk
-    import glib
-    import cairo
-    import ninix.pix
+import gtk
+import glib
+import cairo
 
+import ninix.pix
 import ninix.home
 
-class Menu:
 
-    def __init__(self, callback, accelgroup):
-        self.callback = callback
+class Menu(object):
+
+    def __init__(self, accelgroup):
+        self.request_parent = lambda *a: None # dummy
         ui_info = '''
         <ui>
           <popup name='popup'>
@@ -74,19 +75,17 @@ class Menu:
         '''
         self.__menu_list = {
             'settings': [('Settings', None, _('Settings...(_O)'), None,
-                          '', self.callback['edit_preferences']),
+                          '', lambda *a: self.request_parent('NOTIFY', 'edit_preferences')),
                          '/ui/popup/Settings'],
-            'katochan': [('Katochan', None,_('Katochan(_K)'), None),
+            'katochan': [('Katochan', None, _('Katochan(_K)'), None),
                          '/ui/popup/Katochan'],
             'exit':     [('Exit', None,_('Exit(_Q)'), None,
-                          '', self.callback['close']),
+                          '', lambda *a: self.request_parent('NOTIFY', 'close')),
                          '/ui/popup/Exit'],
             }
         self.__katochan_list = None
         actions = gtk.ActionGroup('Actions')
-        entry = []
-        for value in self.__menu_list.itervalues():
-            entry.append(value[0])
+        entry = [value[0] for value in self.__menu_list.values()]
         actions.add_actions(tuple(entry))
         ui_manager = gtk.UIManager()
         ui_manager.insert_action_group(actions, 0)
@@ -96,8 +95,11 @@ class Menu:
             path = self.__menu_list[key][-1]
             self.__menu_list[key][1] = ui_manager.get_widget(path)
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def popup(self, button):
-        katochan_list = self.callback['get_katochan_list']()
+        katochan_list = self.request_parent('GET', 'get_katochan_list')
         self.__set_katochan_menu(katochan_list)
         self.__popup_menu.popup(
             None, None, None, button, gtk.get_current_event_time())
@@ -109,7 +111,9 @@ class Menu:
             for katochan in list:
                 item = gtk.MenuItem(katochan['name'])
                 item.connect(
-                    'activate', self.callback['select_katochan'], (katochan))
+                    'activate',
+                    lambda a, k: self.request_parent('NOTIFY', 'select_katochan', k),
+                    (katochan))
                 menu.add(item)
                 item.show()
             self.__menu_list[key][1].set_submenu(menu)
@@ -119,7 +123,7 @@ class Menu:
             self.__menu_list[key][1].hide()
 
 
-class Nekoninni:
+class Nekoninni(object):
 
     def __init__(self):
         self.mode = 1 # 0: SEND SSTP1.1, 1: SHIORI/2.2
@@ -127,7 +131,7 @@ class Nekoninni:
         self.skin = None
         self.katochan = None
 
-    def notify_observer(self, event, args): ## FIXME
+    def observer_update(self, event, args):
         if event in ['set position', 'set surface']:
             if self.skin is not None:
                 self.skin.set_position()
@@ -142,7 +146,7 @@ class Nekoninni:
         elif event == 'finalize':
             self.finalize()
         else:
-            #print 'OBSERVER(nekodorif): ignore -', event
+            ##logging.debug('OBSERVER(nekodorif): ignore - {0}'.format(event))
             pass
 
     def load(self, dir, katochan, target):
@@ -150,19 +154,11 @@ class Nekoninni:
             return 0
         self.dir = dir
         self.target = target
-        self.target.set_observer(self)
+        self.target.attach_observer(self)
         self.accelgroup = gtk.AccelGroup()
         scale = self.target.get_surface_scale()
-        skin_callback ={
-            'finalize': self.finalize,
-            'has_katochan': self.has_katochan,
-            'drop_katochan': self.drop_katochan,
-            'edit_preferences': self.edit_preferences,
-            'close': self.close,
-            'get_katochan_list': lambda *a: self.katochan_list,
-            'select_katochan': self.select_katochan,
-            }
-        self.skin = Skin(self.dir, self.accelgroup, skin_callback, scale)
+        self.skin = Skin(self.dir, self.accelgroup, scale)
+        self.skin.set_responsible(self.handle_request)
         if self.skin is None:
             return 0
         self.katochan_list = katochan
@@ -172,6 +168,19 @@ class Nekoninni:
         glib.timeout_add(50, self.do_idle_tasks) # 50ms
         return 1
 
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
+            'get_katochan_list': lambda *a: self.katochan_list,
+            'get_mode': lambda *a: self.mode,
+            }
+        handler = handlers.get(event,
+                               getattr(self, event,
+                                       lambda *a: None)) ## FIXME
+        result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
+
     def do_idle_tasks(self):
         if not self.__running:
             return False
@@ -194,14 +203,12 @@ class Nekoninni:
                 self.katochan.get_category(),
                 self.katochan.get_kinoko_flag(),
                 self.katochan.get_target())
-        ##print 'KATOCHAN:', args
-        ##print 'TARGET:', args[4]
-        self.target.notify_event('OnNekodorifObject%s' % event, *args)
+        self.target.notify_event('OnNekodorifObject{0}'.format(event), *args)
 
     def has_katochan(self):
         return int(self.katochan is not None)
 
-    def select_katochan(self, widget, args):
+    def select_katochan(self, args):
         self.launch_katochan(args)
 
     def drop_katochan(self):
@@ -215,21 +222,16 @@ class Nekoninni:
     def launch_katochan(self, katochan):
         if self.katochan:
             self.katochan.destroy()
-        katochan_callback = {
-            'finalize': self.finalize,
-            'send_event': self.send_event,
-            'get_mode': lambda *a: self.mode,
-            'delete_katochan': self.delete_katochan,
-            }
-        self.katochan = Katochan(katochan_callback, self.target)
+        self.katochan = Katochan(self.target)
+        self.katochan.set_responsible(self.handle_request)
         self.katochan.load(katochan)
 
-    def edit_preferences(self, action):
+    def edit_preferences(self):
         pass
 
     def finalize(self):
         self.__running = 0
-        self.target.delete_observer(self)
+        self.target.detach_observer(self)
         if self.katochan is not None:
             self.katochan.destroy()
         if self.skin is not None:
@@ -237,24 +239,22 @@ class Nekoninni:
         ##if self.balloon is not None:
         ##    self.balloon.destroy()
 
-    def close(self, action):
+    def close(self):
         self.finalize()
 
 
-class Skin:
+class Skin(object):
 
-    def __init__(self, dir, accelgroup, callback, scale):
+    def __init__(self, dir, accelgroup, scale):
         self.dir = dir
         self.accelgroup = accelgroup
-        self.callback = callback
+        self.request_parent = lambda *a: None # dummy
+        self.dragged = False
         self.x_root = None
         self.y_root = None
         self.__scale = scale
-        menu_callback = {}
-        for key in ['edit_preferences', 'close',
-                    'get_katochan_list', 'select_katochan']:
-            menu_callback[key] = self.callback[key]
-        self.__menu = Menu(menu_callback, self.accelgroup)
+        self.__menu = Menu(self.accelgroup)
+        self.__menu.set_responsible(self.handle_request)
         path = os.path.join(self.dir, 'omni.txt')
         self.omni = int(os.path.isfile(path) and os.path.getsize(path) == 0)
         self.window = ninix.pix.TransparentWindow()
@@ -264,6 +264,7 @@ class Skin:
         self.window.set_resizable(False)
         self.window.set_focus_on_map(False)
         self.window.connect('delete_event', self.delete)
+        self.window.connect('key_press_event', self.key_press)
         self.window.realize()
         self.window.window.set_back_pixmap(None, False)
         self.window.add_accel_group(self.accelgroup)
@@ -287,6 +288,22 @@ class Skin:
         self.set_position(reset=1)
         self.window.show()
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
+            }
+        handler = handlers.get(event, getattr(self, event, None))
+        if handler is None:
+            result = self.request_parent(
+                event_type, event, *arglist, **argdict)
+        else:
+            result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
+
     def set_scale(self, scale):
         self.__scale = scale
         self.set_surface()
@@ -304,24 +321,28 @@ class Skin:
         del cr
 
     def delete(self, widget, event):
-        self.callback['finalize']()
+        self.request_parent('NOTIFY', 'finalize')
+
+    def key_press(self, window, event):
+        if event.state & (gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK):
+            if event.keyval == gtk.keysyms.F12:
+                logging.info('reset skin position')
+                self.set_position(reset=1)
+        return True
 
     def destroy(self): ## FIXME
         self.window.destroy()
 
-    def button_press(self,  widget, event): ## FIXME
+    def button_press(self,  widget, event):
         self.x_root = event.x_root
         self.y_root = event.y_root
         if event.button == 1:
             if event.type == gtk.gdk.BUTTON_PRESS:
-                ##self.window.begin_move_drag(
-                ##    event.button, int(event.x_root), int(event.y_root),
-                ##    gtk.get_current_event_time())
                 pass
             elif event.type == gtk.gdk._2BUTTON_PRESS: # double click
-                if self.callback['has_katochan']():
+                if self.request_parent('GET', 'has_katochan'):
                     self.start() ## FIXME
-                    self.callback['drop_katochan']()
+                    self.request_parent('NOTIFY', 'drop_katochan')
         elif event.button == 3:                
             if event.type == gtk.gdk.BUTTON_PRESS:
                 self.__menu.popup(event.button)
@@ -329,14 +350,16 @@ class Skin:
 
     def set_surface(self):
         if self.id[1] is not None:
-            path = os.path.join(self.dir,
-                                'surface%d%d.png' % (self.id[0], self.id[1]))
+            path = os.path.join(
+                self.dir,
+                'surface{0:d}{1:d}.png'.format(self.id[0], self.id[1]))
             if not os.path.exists(path):
                 self.id[1] = None
                 self.set_surface()
                 return
         else:
-            path = os.path.join(self.dir, 'surface%d.png' % self.id[0])
+            path = os.path.join(self.dir,
+                                'surface{0:d}.png'.format(self.id[0]))
         try:
             surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
             w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
@@ -344,18 +367,14 @@ class Skin:
             surface_pixbuf = surface_pixbuf.scale_simple(
                 w, h, gtk.gdk.INTERP_BILINEAR)
         except:
-            self.callback['finalize']()
+            self.request_parent('NOTIFY', 'finalize')
             return
         self.w, self.h = w, h
         mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
         surface_pixbuf.render_threshold_alpha(
             mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        if self.window.is_composited():
-            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
-        else:
-            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.darea.set_size_request(self.w, self.h)
-        self.current_mask = mask_pixmap
+        self.window.mask = mask_pixmap
         self.current_surface_pixbuf = surface_pixbuf
         self.darea.queue_draw()
 
@@ -365,13 +384,11 @@ class Skin:
             self.x = left
             self.y = top + scrn_h - self.h
         else:
-            self.x = min(left + scrn_w - self.w, self.x)
-            self.x = max(left, self.x)
-            self.y = min(top + scrn_h - self.h, self.y)
-            self.y = max(top, self.y)
-        self.window.move(self.x, self.y) ## FIXME
+            if not self.omni:
+                self.y = top + scrn_h - self.h
+        self.window.resize_move(self.x, self.y)
 
-    def move(self, x_delta, y_delta): ## FIXME
+    def move(self, x_delta, y_delta):
         self.x = self.x + x_delta
         if self.omni:
             self.y = self.y + y_delta
@@ -394,6 +411,9 @@ class Skin:
         self.set_surface()
 
     def button_release(self,  widget, event):
+        if self.dragged:
+            self.dragged = False
+            self.set_position()
         self.x_root = None
         self.y_root = None
         return True
@@ -406,6 +426,7 @@ class Skin:
         if state & gtk.gdk.BUTTON1_MASK:
             if self.x_root is not None and \
                self.y_root is not None:
+                self.dragged = True
                 x_delta = int(event.x_root - self.x_root)
                 y_delta = int(event.y_root - self.y_root)
                 self.move(x_delta, y_delta)
@@ -417,7 +438,7 @@ class Skin:
         pass
 
 
-class Balloon:
+class Balloon(object):
 
     def __init__(self):
         pass
@@ -426,7 +447,7 @@ class Balloon:
         pass
 
 
-class Katochan:
+class Katochan(object):
 
     CATEGORY_LIST = ['pain',      # 痛い
                      'stab',      # 刺さる
@@ -441,10 +462,10 @@ class Katochan:
                      'other'      # 上記カテゴリに当てはまらないもの
                      ]
 
-    def __init__(self, callback, target):
+    def __init__(self, target):
         self.side = 0
         self.target = target
-        self.callback = callback
+        self.request_parent = lambda *a: None # dummy
         self.settings = {}
         self.settings['state'] = 'before'
         self.settings['fall.type'] = 'gravity'
@@ -457,6 +478,9 @@ class Katochan:
         self.__scale = 100
         self.loaded = False
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def get_name(self):
         return self.data['name']
 
@@ -536,10 +560,10 @@ class Katochan:
         self.x = target_x + target_w / 2 - self.w / 2 + \
                  self.offset_x * self.__scale / 100
         self.y = top + self.offset_y * self.__scale / 100
-        self.window.move(self.x, self.y) ## FIXME
+        self.window.resize_move(self.x, self.y)
 
     def set_surface(self):
-        path = os.path.join(self.data['dir'], 'surface%d.png' % self.id)
+        path = os.path.join(self.data['dir'], 'surface{0:d}.png'.format(self.id))
         try:
             surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
             w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
@@ -547,19 +571,15 @@ class Katochan:
             surface_pixbuf = surface_pixbuf.scale_simple(
                 w, h, gtk.gdk.INTERP_BILINEAR)
         except:
-            self.callback['finalize']()
+            self.request_parent('NOTIFY', 'finalize')
             return
         self.w, self.h = w, h
         mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
         surface_pixbuf.render_threshold_alpha(
             mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        if self.window.is_composited():
-            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
-        else:
-            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.darea.set_size_request(self.w, self.h)
         #self.darea.queue_draw_area(0, 0, self.w, self.h)
-        self.current_mask = mask_pixmap
+        self.window.mask = mask_pixmap
         self.current_surface_pixbuf = surface_pixbuf
         self.darea.queue_draw()
 
@@ -567,12 +587,11 @@ class Katochan:
         self.data = data
         self.__scale = self.target.get_surface_scale()
         self.set_state('before')
-        ##print 'DATA:', self.data
         if 'category' in self.data:
             category = self.data['category'].split(',')
             if category:
                 if category[0] not in self.CATEGORY_LIST:
-                    print 'WARNING: unknown major category - ', category[0]
+                    logging.warning('WARNING: unknown major category - {0}'.format(category[0]))
                     ##self.data['category'] = self.CATEGORY_LIST[-1]
             else:
                 self.data['category'] = self.CATEGORY_LIST[-1]
@@ -587,8 +606,8 @@ class Katochan:
                 self.side = 0 # XXX
         else:
             self.side = 0 # XXX
-        if self.callback['get_mode']() == 1:
-            self.callback['send_event']('Emerge')
+        if self.request_parent('GET', 'get_mode') == 1:
+            self.request_parent('NOTIFY', 'send_event', 'Emerge')
         else:
             if 'before.script' in self.data:
                 pass ## FIXME
@@ -645,7 +664,6 @@ class Katochan:
         pass
 
     def update_position(self): ## FIXME
-        ##print 'MOVE:', self.settings
         if self.settings['slide.type'] == 'leaf':
             pass
         else:
@@ -660,8 +678,7 @@ class Katochan:
                 pass ## FIXME
             else:
                 pass
-        ##print 'POSITION:', self.x, self.y
-        self.window.move(self.x, self.y)
+        self.window.resize_move(self.x, self.y)
 
     def check_collision(self): ## FIXME: check self position
         for side in [0, 1]:
@@ -693,10 +710,10 @@ class Katochan:
             if self.check_collision():
                 self.set_state('hit')
                 self.hit = 1
-                if self.callback['get_mode']() == 1:
+                if self.request_parent('GET', 'get_mode') == 1:
                     self.id = 1
                     self.set_surface()
-                    self.callback['send_event']('Hit')
+                    self.request_parent('NOTIFY', 'send_event', 'Hit')
                 else:
                     pass ## FIXME
             if self.check_mikire():
@@ -705,10 +722,10 @@ class Katochan:
             if self.hit_stop >= self.data.get('hit.waittime', 0):
                 self.set_state('after')
                 self.set_movement('after')
-                if self.callback['get_mode'] == 1:
+                if self.request_parent('GET', 'get_mode') == 1:
                     self.id = 2
                     self.set_surface()
-                    self.callback['send_event']('Drop')
+                    self.request_parent('NOTIFY', 'send_event', 'Drop')
                 else:
                     pass ## FIXME
             else:
@@ -720,18 +737,18 @@ class Katochan:
             if self.check_mikire():
                 self.set_state('end')
         elif self.settings['state'] == 'end':
-            if self.callback['get_mode']() == 1:
-                self.callback['send_event']('Vanish')
+            if self.request_parent('GET', 'get_mode') == 1:
+                self.request_parent('NOTIFY', 'send_event', 'Vanish')
             else:
                 pass ## FIXME
-            self.callback['delete_katochan']()
+            self.request_parent('NOTIFY', 'delete_katochan')
             return False
         elif self.settings['state'] == 'dodge':
-            if self.callback['get_mode'] == 1:
-                self.callback['send_event']('Dodge')
+            if self.request_parent('GET', 'get_mode') == 1:
+                self.request_parent('NOTIFY', 'send_event', 'Dodge')
             else:
                 pass ## FIXME
-            self.callback['delete_katochan']()
+            self.request_parent('NOTIFY', 'delete_katochan')
             return False
         else:
             pass ## check collision and mikire
