@@ -22,8 +22,8 @@ import gobject
 import ninix.home
 
 
-range_scale = range(100, 30, -10) + [200, 300, 1000]
-range_script_speed = range(-1, 5) + [6, 8] # -1: no wait
+range_scale = list(range(100, 30, -10)) + [200, 300, 1000]
+range_script_speed = list(range(-1, 5)) + [6, 8] # -1: no wait
 
 # default settings
 DEFAULT_BALLOON_FONTS = 'Sans'
@@ -40,28 +40,43 @@ class Preferences(dict):
     def __init__(self, filename):
         dict.__init__(self)
         self.filename = filename
+        self.__stack = {}
+
+    def __setitem__(self, key, item):
+        if key in self and key not in self.__stack:
+            self.__stack[key] = self[key]
+        dict.__setitem__(self, key, item)
+
+    def commit(self):
+        self.__stack = {}
+
+    def revert(self):
+        self.update(self.__stack)
+        self.__stack = {}
 
     def load(self):
+        self.clear()
         try:
-            f = open(self.filename)
+            with open(self.filename) as f:
+                prefs = mimetools.Message(f)
+                for key, value in prefs.items():
+                    self[key] = value
         except IOError:
             return
-        prefs = mimetools.Message(f)
-        for key, value in prefs.items():
-            self[key] = value
-        f.close()
 
     def save(self):
         try:
             os.makedirs(os.path.dirname(self.filename))
         except OSError:
             pass
-        f = open(self.filename, 'w')
-        keys = self.keys()
-        keys.sort()
-        for key in keys:
-            f.write('%s: %s\n' % (key, self[key]))
-        f.close()
+        with open(self.filename, 'w') as f:
+            keys = sorted(self.keys())
+            for key in keys:
+                if key in self.__stack:
+                    value = self.__stack[key]
+                else:
+                    value = self[key]
+                f.write('{0}: {1}\n'.format(key, value))
 
     def get_with_type(self, name, conv, default):
         value = self.get(name)
@@ -75,10 +90,11 @@ class Preferences(dict):
         return default
 
 
-class PreferenceDialog:
+class PreferenceDialog(object):
 
-    PREFS_TYPE = {'sakura_name': None,
-                  'sakura_surface': int,
+    PREFS_TYPE = {'sakura_name': None, # XXX: backward compat
+                  'sakura_dir': None,
+                  'sakura_surface': None,
                   'default_balloon': None,
                   'ignore_default': int,
                   'script_speed': int,
@@ -96,8 +112,8 @@ class PreferenceDialog:
                   'seriko_inactive': int,
                   }
 
-    def __init__(self, callback):
-        self.callback = callback
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.window = gtk.Dialog()
         self.window.set_title('Preferences')
         self.window.connect('delete_event', self.cancel)
@@ -130,6 +146,9 @@ class PreferenceDialog:
         box.add(button)
         button.show()
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def load(self):
         filename = ninix.home.get_preferences()
         self.__prefs = Preferences(filename)
@@ -137,9 +156,8 @@ class PreferenceDialog:
         self.__prefs.load()
         self.reset()
         self.update() # XXX
-        self.__saved_prefs = self.__prefs.copy()
-        self.callback['notify_preference_changed']()
-        
+        self.request_parent('NOTIFY', 'notify_preference_changed')
+ 
     def reset(self): ### FIXME ###
         self.fontsel.set_font_name(
             self.get('balloon_fonts', DEFAULT_BALLOON_FONTS))
@@ -175,11 +193,18 @@ class PreferenceDialog:
         assert name in self.PREFS_TYPE
         return self.__prefs.get_with_type(name, self.PREFS_TYPE[name], default)
 
-    def set_current_sakura(self, name, surface):
-        key = 'sakura_name'
-        self.__prefs[key] = self.__saved_prefs[key] = name
+    def set_current_sakura(self, directory, surface):
+        key = 'sakura_name' # obsolete
+        if key in self.__prefs:
+            del self.__prefs[key]
+        key = 'sakura_dir'
+        if key in self.__prefs:
+            del self.__prefs[key]
+        self.__prefs[key] = directory
         key = 'sakura_surface'
-        self.__prefs[key] = self.__saved_prefs[key] = surface
+        if key in self.__prefs:
+            del self.__prefs[key]
+        self.__prefs[key] = surface
 
     def edit_preferences(self):
         self.show()
@@ -208,18 +233,19 @@ class PreferenceDialog:
     def ok(self, widget):
         self.hide()
         self.update()
-        self.__saved_prefs.update(self.__prefs)
-        self.callback['notify_preference_changed']()
+        self.__prefs.commit()
+        self.request_parent('NOTIFY', 'notify_preference_changed')
 
     def apply(self, widget):
         self.update()
-        self.callback['notify_preference_changed']()
+        self.request_parent('NOTIFY', 'notify_preference_changed')
+
 
     def cancel(self, widget, event=None):
         self.hide()
-        self.__prefs.update(self.__saved_prefs)
+        self.__prefs.revert()
         self.reset()
-        self.callback['notify_preference_changed']()
+        self.request_parent('NOTIFY', 'notify_preference_changed')
         return True
 
     def show(self):
@@ -262,7 +288,7 @@ class PreferenceDialog:
         label.show()
         self.surface_scale_combo = gtk.combo_box_new_text()
         for value in range_scale:
-            self.surface_scale_combo.append_text('%4d%%' % value)
+            self.surface_scale_combo.append_text('{0:4d}%'.format(value))
         hbox.pack_start(self.surface_scale_combo, False)
         self.surface_scale_combo.show()
         button = gtk.CheckButton(unicode(_('Scale Balloon'), 'utf-8'))
@@ -337,7 +363,7 @@ class PreferenceDialog:
         hbox = gtk.HBox(spacing=5)
         box.add(hbox)
         hbox.show()
-        label = gtk.Label(unicode(_("Quality"), 'utf-8'))
+        label = gtk.Label(unicode(_('Quality'), 'utf-8'))
         hbox.pack_start(label, False)
         label.show()        
         self.animation_quality_adjustment = gtk.Adjustment(1.0, 0.1, 1.0, 0.1, 0.1)
@@ -420,7 +446,7 @@ class PreferenceDialog:
 
     def set_default_balloon(self, directory):
         model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-        for name, directory in self.callback['get_balloon_list']():
+        for name, directory in self.request_parent('GET', 'get_balloon_list'):
             listiter = model.append()
             model.set_value(listiter, 0, name)
             model.set_value(listiter, 1, directory)
