@@ -15,6 +15,7 @@
 import re
 import sys
 
+
 TOKEN_TAG         = 1
 TOKEN_META        = 2
 TOKEN_OPENED_SBRA = 3
@@ -23,8 +24,8 @@ TOKEN_NUMBER      = 5
 TOKEN_STRING      = 6
 
 patterns = [
-    (TOKEN_TAG, re.compile(r'\\[ehunjcxtpqzy*v0123456789fmia!&+---]|'
-                           r'\\[sb][0-9]?|\\w[0-9]|\\_[wqslvVbe+cumna]|'
+    (TOKEN_TAG, re.compile(r'\\[ehunjcxtqzy*v0123456789fmia!&+---]|'
+                           r'\\[sbp][0-9]?|\\w[0-9]|\\_[wqslvVbe+cumna]|'
                            r'\\__[ct]|\\URL')),
     (TOKEN_META, re.compile(r'%month|%day|%hour|%minute|%second|%username|'
                             r'%selfname2?|%keroname|%friendname|%songname|'
@@ -49,7 +50,7 @@ class ParserError(Exception):
     def __init__(self, message, error='strict',
                  script=None, src=None, column=None, length=None, skip=None):
         if error not in ['strict', 'loose']:
-            raise ValueError, 'unknown error scheme: %s' % str(error)
+            raise ValueError, 'unknown error scheme: {0}'.format(str(error))
         self.message = message
         self.error = error
         self.script = script or []
@@ -86,20 +87,21 @@ class ParserError(Exception):
         else:
             column = '??'
             dump = self.src
-        return 'ParserError: column %s: %s\n%s' % (column, self.message, dump)
+        return 'ParserError: column {0}: {1}\n{2}'.format(column, self.message, dump)
 
-class Parser:
+
+class Parser(object):
 
     def __init__(self, error='strict'):
         if error not in ['strict', 'loose']:
-            raise ValueError, 'unknown error scheme: %s' % str(error)
+            raise ValueError, 'unknown error scheme: {0}'.format(str(error))
         self.error = error
 
     def perror(self, msg, position='column', skip=None):
         if position not in ['column', 'eol']:
-            raise ValueError, 'unknown position scheme: %s' % str(position)
+            raise ValueError, 'unknown position scheme: {0}'.format(str(position))
         if skip not in ['length', 'rest', None]:
-            raise ValueError, 'unknown skip scheme: %s' % str(skip)
+            raise ValueError, 'unknown skip scheme: {0}'.format(str(skip))
         if position == 'column':
             column = self.column
             length = self.length
@@ -141,7 +143,8 @@ class Parser:
         return token, lexeme
 
     def parse(self, s):
-        if not s: return []
+        if not s:
+            return []
         # tokenize the script
         self.src = s
         self.tokens = self.tokenize(self.src)
@@ -271,7 +274,7 @@ class Parser:
                         self.split_params(self.read_sbra_text())]
                 self.script.append((SCRIPT_TAG, lexeme) + tuple(args))
             else:
-                raise self.perror('unknown tag (%s)' % lexeme, skip='length')
+                raise self.perror('unknown tag ({0})'.format(lexeme), skip='length')
         if anchor:
             if self.script[-1] == (SCRIPT_TAG, r'\e'):
                 self.script.insert(len(self.script) - 1, (SCRIPT_TAG, r'\_a'))
@@ -416,7 +419,7 @@ testcases = [
     r'\![raise,OnWavePlay,voice\hello.mp3]\e',
     r'\q[Asahi.com,新聞を読む]',
     r'\j[\s4]\e',
-    r'\p[2]\s[100]3人目',
+    r'\p[2]\s[100]3人目\p3\s[0]4人目',
     r'\_s[0,2]keroは\_s仲間はずれ\_sです。\e',
     # illegal cases (to be passed)
     r'20%終了 (%hour時%minute分%second秒)',
@@ -442,7 +445,7 @@ def test_tokenizer():
     for test in testcases:
         try:
             print parser.tokenize(test)
-        except ParserError, e:
+        except ParserError as e:
             print e
 
 def test_parser(error='strict'):
@@ -454,7 +457,7 @@ def test_parser(error='strict'):
         while 1:
             try:
                 script.extend(parser.parse(test))
-            except ParserError, e:
+            except ParserError as e:
                 print '-' * 60
                 print e
                 done, test = e
@@ -471,9 +474,9 @@ def print_script_tree(tree):
             print 'TAG', name
             for n in range(len(args)):
                 if isinstance(args[n], str):
-                    print '\tARG#%d\t%s' % (n + 1, args[n])
+                    print '\tARG#{0:d}\t{1}'.format(n + 1, args[n])
                 else:
-                    print '\tARG#%d\tTEXT' % (n + 1)
+                    print '\tARG#{0:d}\tTEXT'.format(n + 1)
                     print_text(args[n], 2)
         elif node[0] == SCRIPT_TEXT:
             print 'TEXT'
@@ -482,12 +485,13 @@ def print_script_tree(tree):
 def print_text(text, indent):
     for chunk in text:
         if chunk[0] == TEXT_STRING:
-            print ''.join(('\t' * indent, 'STRING\t"%s"' % chunk[1]))
+            print ''.join(('\t' * indent, 'STRING\t"{0}"'.format(chunk[1])))
         elif chunk[0] == TEXT_META:
             name, args = chunk[1], chunk[2:]
             print ''.join(('\t' * indent, 'META\t', name))
             for n in range(len(args)):
-                print ''.join(('\t' * indent, '\tARG#%d\t%s' % (n + 1, args[n])))
+                print ''.join(('\t' * indent, '\tARG#{0:d}\t{1}'.format(n + 1, args[n])))
+
 
 if __name__ == '__main__':
     import os
